from __future__ import annotations
from ..Base import *
from ..Proxy import *
from ..AcadObject import *



class AcadSelectionSet(AppObject):
    def __init__(self, obj) -> None: super().__init__(obj)

    Application: AcadApplication = proxy_property('AcadApplication','Application',AccessMode.ReadOnly)
    Count: int = proxy_property(int,'Count',AccessMode.ReadOnly)
    Name: str = proxy_property(str,'Name',AccessMode.ReadOnly)

    def AddItems(self, Items: vObjectArray) -> None:
        self._obj.AddItems(Items())

    def Clear(self) -> None:
        self._obj.Clear()

    def Delete(self) -> None:
        self._obj.Delete()

    def Erase(self) -> None:
        self._obj.Erase()

    def Highlight(self, HighlightFlag: bool) -> None:
        self._obj.Highlight(HighlightFlag)

    def Item(self, Index: int | str) -> AcadObject:
        return AcadObject(self._obj.Item(Index))

    def RemoveItems(self, Objects: vObjectArray) -> None:
        self._obj.RemoveItems(Objects())

    def Select(self, Mode: AcSelect, Point1: PyGePoint3d = None, Point2: PyGePoint3d = None, FilterType: Variant[AdeskDxfCode] = None, FilterData: Variant = None) -> None:
        kwargs = {'Mode':Mode}
        if Point1 is not None: kwargs.update({'Point1':Point1()}) 
        if Point2 is not None: kwargs.update({'Point2':Point2()}) 
        if (FilterType is not None) and (FilterData is not None): kwargs.update({'FilterType':FilterType(),'FilterData':FilterData()}) 
        self._obj.Select(*kwargs)

    def SelectAtPoint(self, Point: PyGePoint3d, FilterType: Variant[AdeskDxfCode] = None, FilterData: Variant = None) -> None:
        kwargs = {'Point':Point()}
        if (FilterType is not None) and (FilterData is not None): kwargs.update({'FilterType':FilterType(),'FilterData':FilterData()}) 
        self._obj.SelectAtPoint(*kwargs)

    def SelectByPolygon(self, Mode: AcSelect, PointsList: PyGePoint3dArray, FilterType: Variant[AdeskDxfCode] = None, FilterData: Variant = None) -> None:
        kwargs = {'Mode':Mode,'PointsList':PointsList()}
        if (FilterType is not None) and (FilterData is not None): kwargs.update({'FilterType':FilterType(),'FilterData':FilterData()}) 
        self._obj.SelectByPolygon(*kwargs)

    def SelectOnScreen(self, FilterType: Variant[AdeskDxfCode] = None, FilterData: Variant = None) -> None:
        if (FilterType is not None) and (FilterData is not None):
            self._obj.SelectOnScreen(FilterType(), FilterData())
        else:
            self._obj.SelectOnScreen()

    def Update(self) -> None:
        self._obj.Update()


from ..Base import *
from ..Proxy import *
from .AcadDimension import AcadDimension as AcadDimension

class AcadDimOrdinate(AcadDimension):
    def __init__(self, obj) -> None: ...
    AltRoundDistance: float
    AltSubUnitsFactor: float
    AltSubUnitsSuffix: str
    AltSuppressLeadingZeros: bool
    AltSuppressTrailingZeros: bool
    AltSuppressZeroFeet: bool
    AltSuppressZeroInches: bool
    AltTextPrefix: str
    AltTextSuffix: str
    AltTolerancePrecision: AcDimPrecision
    AltToleranceSuppressLeadingZeros: bool
    AltToleranceSuppressTrailingZeros: bool
    AltToleranceSuppressZeroFeet: bool
    AltToleranceSuppressZeroInches: bool
    AltUnits: bool
    AltUnitsFormat: AcDimUnits
    AltUnitsPrecision: AcDimPrecision
    AltUnitsScale: float
    ArrowheadSize: int
    ExtensionLineColor: AcColor
    ExtensionLineOffset: float
    ExtensionLineWeight: AcLineWeight
    ExtLineFixedLen: float
    ExtLineFixedLenSuppress: bool
    FractionFormat: AcDimFractionType
    LinearScaleFactor: float
    Measurement: float
    PrimaryUnitsPrecision: AcDimPrecision
    RoundDistance: float
    SubUnitsFactor: float
    SubUnitsSuffix: str
    SuppressZeroFeet: bool
    SuppressZeroInches: bool
    ToleranceSuppressZeroFeet: bool
    ToleranceSuppressZeroInches: bool
    UnitsFormat: AcDimLUnits
    def Copy(self) -> AcadDimOrdinate: ...



from __future__ import annotations
from ..Base import *
from ..Proxy import *
from ..AcadEntity import AcadEntity



class AcadOle(AcadEntity):
    def __init__(self, obj) -> None: super().__init__(obj)

    Height: float = proxy_property(float,'Height',AccessMode.ReadWrite)
    InsertionPoint: PyGePoint3d = proxy_property('PyGePoint3d','InsertionPoint',AccessMode.ReadWrite)
    LockAspectRatio: bool = proxy_property(bool,'LockAspectRatio',AccessMode.ReadWrite)
    OleItemType: AcOleType = proxy_property('AcOleType','OleItemType',AccessMode.ReadWrite)
    OlePlotQuality: AcOlePlotQuality = proxy_property('AcOlePlotQuality','OlePlotQuality',AccessMode.ReadWrite)
    OleSourceApp: str = proxy_property(str,'OleSourceApp',AccessMode.ReadWrite)
    Rotation: float = proxy_property(float,'Rotation',AccessMode.ReadWrite)
    ScaleHeight: float = proxy_property(float,'ScaleHeight',AccessMode.ReadWrite)
    ScaleWidth: float = proxy_property(float,'ScaleWidth',AccessMode.ReadWrite)
    Width: float = proxy_property(float,'Width',AccessMode.ReadWrite)

    def Copy(self) -> AcadOle:
        return AcadOle(self._obj.Copy())



from __future__ import annotations
from ..Base import *
from ..Proxy import *
from .AcadSurface import AcadSurface



class AcadSweptSurface(AcadSurface):
    def __init__(self, obj) -> None: super().__init__(obj)

    Bank: bool = proxy_property(bool,'Bank',AccessMode.ReadWrite)
    ProfileRotation: float = proxy_property(float,'ProfileRotation',AccessMode.ReadWrite)
    Scale: float = proxy_property(float,'Scale',AccessMode.ReadWrite)
    Twist: float = proxy_property(float,'Twist',AccessMode.ReadWrite)

    def Copy(self) -> AcadSweptSurface:
        return AcadSweptSurface(self._obj.Copy())


from ..Base import *
from ..AcadObject import *
from ..Proxy import AccessMode as AccessMode, proxy_property as proxy_property

class AcadLineType(AcadObject):
    def __init__(self, obj) -> None: ...
    Description: str
    Name: str



from __future__ import annotations
from ..Base import *
from ..Proxy import *
from ..AcadObject import *



class AcadSectionSettings(AcadObject):
    def __init__(self, obj) -> None: super().__init__(obj)

    CurrentSectionType: AcSectionType = proxy_property('AcSectionType','CurrentSectionType',AccessMode.ReadWrite)



from ..Base import *
from ..Proxy import *
from ..AcadEntity import AcadEntity as AcadEntity

class AcadPolyline(AcadEntity):
    def __init__(self, obj) -> None: ...
    Area: float
    Closed: bool
    ConstantWidth: float
    Coordinates: PyGePoint3dArray
    Elevation: float
    Length: float
    LinetypeGeneration: bool
    Normal: PyGeVector3d
    Thickness: float
    Type: AcPolylineType
    def Coordinate(self, Index: int) -> PyGePoint3d: ...
    def Copy(self) -> AcadPolyline: ...
    def AppendVertex(self, Point: PyGePoint3d) -> None: ...
    def Explode(self) -> vObjectArray: ...
    def GetBulge(self, Index: int) -> float: ...
    def GetWidth(self, Index: int) -> vDoubleArray: ...
    def Offset(self, Distance: float) -> vObjectArray: ...
    def SetBulge(self, Index: int, Value: float) -> None: ...
    def SetWidth(self, SegmentIndex: int, StartWidth: float, EndWidth: float) -> None: ...



from __future__ import annotations
from ..Base import *
from ..Proxy import *
from ..AcadEntity import AcadEntity



class AcadCircle(AcadEntity):
    def __init__(self, obj) -> None: super().__init__(obj)

    Area: float = proxy_property(float,'Area',AccessMode.ReadWrite)
    Center: PyGePoint3d = proxy_property('PyGePoint3d','Center',AccessMode.ReadWrite)
    Circumference: float = proxy_property(float,'Circumference',AccessMode.ReadWrite)
    Diameter: float = proxy_property(float,'Diameter',AccessMode.ReadWrite)
    Normal: PyGeVector3d = proxy_property('PyGeVector3d','Normal',AccessMode.ReadWrite)
    Radius: float = proxy_property(float,'Radius',AccessMode.ReadWrite)
    Thickness: float = proxy_property(float,'Thickness',AccessMode.ReadWrite)

    def Copy(self) -> AcadCircle:
        return AcadCircle(self._obj.Copy())

    def Offset(self, Distance: float) -> vObjectArray:
        return vObjectArray(self._obj.Offset(Distance))


from ..Base import *
from ..Proxy import *
from ..AcadEntity import AcadEntity as AcadEntity

class AcadAttributeReference(AcadEntity):
    def __init__(self, obj) -> None: ...
    Alignment: AcAlignment
    Backward: bool
    Constant: bool
    FieldLength: int
    Height: float
    InsertionPoint: PyGePoint3d
    Invisible: bool
    LockPosition: bool
    MTextAttribute: bool
    MTextAttributeContent: str
    MTextBoundaryWidth: float
    MTextDrawingDirection: AcDrawingDirection
    Normal: PyGeVector3d
    ObliqueAngle: float
    Rotation: float
    ScaleFactor: float
    StyleName: str
    TagString: str
    TextAlignmentPoint: PyGePoint3d
    TextGenerationFlag: AcTextGenerationFlag
    TextString: str
    Thickness: float
    UpsideDown: bool
    def Copy(self) -> AcadAttributeReference: ...
    def UpdateMTextAttribute(self) -> None: ...



from ..Base import *
from ..Proxy import *
import abc
from ..AcadEntity import AcadEntity as AcadEntity
from abc import ABC, abstractmethod

class AcadUnderlay(AcadEntity, ABC, metaclass=abc.ABCMeta):
    def __init__(self, obj) -> None: ...
    AdjustForBackground: bool
    ClippingEnabled: bool
    Contrast: int
    Fade: int
    File: str
    Height: float
    ItemName: str
    Monochrome: bool
    Position: PyGePoint3d
    Rotation: float
    ScaleFactor: float
    UnderlayLayerOverrideApplied: bool
    UnderlayName: str
    UnderlayVisibility: bool
    Width: float
    def ClipBoundary(self, PointsArray: PyGePoint3dArray) -> None: ...
    @abstractmethod
    def Copy(self): ...



from ..Base import *
from ..Proxy import *
from ..AcadObject import *

class AcadView(AcadObject):
    def __init__(self, obj) -> None: ...
    CategoryName: str
    Center: PyGePoint2d
    Direction: PyGeVector3d
    HasVpAssociation: bool
    Height: float
    LayerState: str
    LayoutID: int
    Name: str
    Target: PyGePoint3d
    Width: float



from __future__ import annotations
from ..Base import *
from ..Proxy import proxy_property, AccessMode
from ..AcadObject import *
from .AcadSubEntity import *

class AcadMLeaderLeader(AcadSubEntity):
    def __init__(self, obj) -> None: super().__init__(obj)
    
    ArrowheadBlock: str = proxy_property(str,'ArrowheadBlock',AccessMode.ReadWrite)
    ArrowheadSize: int = proxy_property(int,'ArrowheadSize',AccessMode.ReadWrite)
    ArrowheadType: AcDimArrowheadType = proxy_property('AcDimArrowheadType','ArrowheadType',AccessMode.ReadWrite)
    LeaderLineColor: AcadAcCmColor = proxy_property('AcadAcCmColor','LeaderLineColor',AccessMode.ReadWrite)
    LeaderLinetype: str = proxy_property(str,'LeaderLinetype',AccessMode.ReadWrite)
    LeaderLineWeight: AcLineWeight = proxy_property('AcLineWeight','LeaderLineWeight',AccessMode.ReadWrite)
    LeaderType: AcMLeaderType = proxy_property('AcMLeaderType','LeaderType',AccessMode.ReadWrite)



from ..Base import *
from ..AcadObject import *
from ..Proxy import AccessMode as AccessMode, proxy_property as proxy_property
from .AcadViewport import AcadViewport as AcadViewport

class AcadViewports(IAcadObjectCollection):
    def __init__(self, obj) -> None: ...
    def Add(self, Name: str) -> AcadViewport: ...
    def DeleteConfiguration(self, Name: str) -> None: ...
    def Item(self, Index: int | str) -> AcadViewport: ...
    def __iter__(self): ...



from __future__ import annotations
from ..Base import *
from ..Proxy import *
from ..AcadObject import AcadObject



class AcadDimStyle(AcadObject):
    def __init__(self, obj) -> None: super().__init__(obj)

    Name: str = proxy_property(str,'Name',AccessMode.ReadWrite)

    def CopyFrom(self, SourceObject: AcadDimStyle|
                                        AcadDim3PointAngular|
                                        AcadDimAligned|
                                        AcadDimAngular|
                                        AcadDimArcLength|
                                        AcadDimDiametric|
                                        AcadDimOrdinate|
                                        AcadDimRadial|
                                        AcadDimRadialLarge|
                                        AcadDimRotated|
                                        AcadDocument|
                                        AcadLayout|
                                        AcadLeader|
                                        AcadPlotConfiguration) -> None: 
        self._obj.CopyFrom(SourceObject)



from ..Base import *
from ..Proxy import *
from ..AcadObject import *
from ..AcadEntity import *

class AcadSortentsTable(AcadObject):
    def __init__(self, obj) -> None: ...
    def Block(self) -> AcadBlock: ...
    def GetFullDrawOrder(self, honorSortentsSysvar: bool) -> vObjectArray: ...
    def GetRelativeDrawOrder(self, honorSortentsSysvar: bool) -> vObjectArray: ...
    def MoveAbove(self, Target: AcadEntity) -> vObjectArray: ...
    def MoveBelow(self, Target: AcadEntity) -> vObjectArray: ...
    def MoveToBottom(self, Objects: vObjectArray) -> None: ...
    def MoveToTop(self, Objects: vObjectArray) -> None: ...
    def SetRelativeDrawOrder(self, Objects: vObjectArray) -> None: ...
    def SwapOrder(self, Object1: AcadEntity, Object2: AcadEntity) -> None: ...



from __future__ import annotations
from .Vector import *
from .Points import *
import numpy as np
import math

class PyGeMatrix3d:
    """Матрица трансформации 4x4."""

    def __init__(self, matrix_data: PyGeMatrix3d|np.ndarray|None = None):
        # По умолчанию создаем единичную матрицу 4x4 (Identity Matrix)
        if matrix_data is None:
            self.matrix:np.ndarray = np.eye(4, dtype=float)
        elif isinstance(matrix_data, np.ndarray) and matrix_data.shape == (4, 4):
            self.matrix:np.ndarray = matrix_data
        elif isinstance(matrix_data, PyGeMatrix3d):
            self.matrix:np.ndarray = matrix_data.matrix.copy()
        else:
            raise ValueError("Матрица должна быть numpy-массивом 4x4 или PyGeMatrix3d")

    # def __repr__(self):
    #     return f"PyGeMatrix3d(\n{self.matrix}\n)"
    def __str__(self):
        return f"{self.__class__.__name__}(\n{self.matrix}\n)"

    
    @staticmethod
    def translation(vector: PyGeVector3d) -> PyGeMatrix3d:
        """Создает матрицу сдвига (трансляции) по вектору"""
        m = np.eye(4)
        m[:3, 3] = [vector.x, vector.y, vector.z]
        return PyGeMatrix3d(m)

    @staticmethod
    def rotation(angle_rad: float, axis: PyGeVector3d, center: PyGePoint3d = None) -> PyGeMatrix3d:
        """
        Создает матрицу поворота вокруг произвольной оси на заданный угол.
        Если указан center, вращение происходит относительно этой точки.
        """
        axis_vec = np.array([axis.x, axis.y, axis.z])
        # Убедимся, что ось нормализована
        axis_norm = axis_vec / np.linalg.norm(axis_vec)
        ux, uy, uz = axis_norm

        cos_theta = math.cos(angle_rad)
        sin_theta = math.sin(angle_rad)
        one_minus_cos = 1 - cos_theta

        # Формула матрицы вращения Родригеса для произвольной оси:
        R = np.array([
            [cos_theta + ux**2 * one_minus_cos,     ux*uy*one_minus_cos - uz*sin_theta,     ux*uz*one_minus_cos + uy*sin_theta, 0],
            [uy*ux*one_minus_cos + uz*sin_theta,    cos_theta + uy**2 * one_minus_cos,      uy*uz*one_minus_cos - ux*sin_theta, 0],
            [uz*ux*one_minus_cos - uy*sin_theta,    uz*uy*one_minus_cos + ux*sin_theta,     cos_theta + uz**2 * one_minus_cos,  0],
            [0, 0, 0, 1]
        ])
        
        M_rotate = PyGeMatrix3d(R)

        if center is None:
            # Поворот относительно начала координат
            return M_rotate
        else:
            # Поворот относительно произвольного центра (P - C, Rotate, P + C)
            C = np.array([center.x, center.y, center.z])
            T1 = PyGeMatrix3d.translation(PyGeVector3d(-C))
            T2 = PyGeMatrix3d.translation(PyGeVector3d(C))
            
            # Комбинируем матрицы: T2 * R * T1
            combined_matrix = T2.matrix @ M_rotate.matrix @ T1.matrix
            return PyGeMatrix3d(combined_matrix)


    @staticmethod
    def scaling(scale_factor: float, center: PyGePoint3d = None) -> PyGeMatrix3d:
        """Создает матрицу масштабирования. Если указан center, масштабирование происходит относительно этой точки."""

        # 1. Сначала создаем базовую матрицу масштабирования относительно начала координат
        m_scale_base = np.eye(4)
        m_scale_base[0, 0] = scale_factor
        m_scale_base[1, 1] = scale_factor
        m_scale_base[2, 2] = scale_factor
        
        # Оборачиваем ее в объект PyGeMatrix3d
        S = PyGeMatrix3d(m_scale_base)
        
        if center is None:
            # Если центр не указан, возвращаем S
            return S
        else:
            # Если центр указан, комбинируем матрицы сдвига
            C = np.array([center.x, center.y, center.z])
            
            # T1: Сдвиг центра в начало координат
            T1 = PyGeMatrix3d.translation(PyGeVector3d(-C[0], -C[1], -C[2]))
            
            # T2: Обратный сдвиг из начала координат
            T2 = PyGeMatrix3d.translation(PyGeVector3d(C[0], C[1], C[2]))
            
            # Комбинируем матрицы: T2 * S * T1 (помните о порядке умножения матриц!)
            combined_matrix = T2.matrix @ S.matrix @ T1.matrix
            return PyGeMatrix3d(combined_matrix)

    @staticmethod
    def mirroring(plane_origin: PyGePoint3d, plane_normal: PyGeVector3d) -> PyGeMatrix3d:
        """
        Создает матрицу отражения относительно заданной плоскости.
        plane_origin: PyGePoint3d - точка на плоскости.
        plane_normal: PyGeVector3d - вектор нормали к плоскости.
        """
        
        # 1. Нормализуем вектор
        n_vec = np.array([plane_normal.x, plane_normal.y, plane_normal.z])
        n_vec = n_vec / np.linalg.norm(n_vec)
        Nx, Ny, Nz = n_vec
        
        # 2. Базовая матрица отражения (относительно плоскости через начало координат)
        # Формула Родригеса для отражения: I - 2*N*N^T
        M_base_3x3 = np.array([
            [1 - 2*Nx**2,   -2*Nx*Ny,       -2*Nx*Nz],
            [-2*Nx*Ny,      1 - 2*Ny**2,    -2*Ny*Nz],
            [-2*Nx*Nz,      -2*Ny*Nz,       1 - 2*Nz**2]
        ])

        # Преобразуем в гомогенную матрицу 4x4
        M_mirror_base = np.eye(4)
        M_mirror_base[:3, :3] = M_base_3x3
        
        M_mirror = PyGeMatrix3d(M_mirror_base)

        # 3. Комбинация сдвигов, если плоскость не проходит через начало координат
        if plane_origin.x != 0 or plane_origin.y != 0 or plane_origin.z != 0:
            # T1: Сдвигаем начало координат на точку плоскости
            T1 = PyGeMatrix3d.translation(PyGeVector3d(-plane_origin.x, -plane_origin.y, -plane_origin.z))
            # T2: Обратный сдвиг
            T2 = PyGeMatrix3d.translation(PyGeVector3d(plane_origin.x, plane_origin.y, plane_origin.z))
            
            # M = T2 @ M_mirror @ T1
            combined_matrix = T2.matrix @ M_mirror.matrix @ T1.matrix
            return PyGeMatrix3d(combined_matrix)
        
        return M_mirror
    
    @staticmethod
    def projection(plane_origin: PyGePoint3d, plane_normal: PyGeVector3d) -> PyGeMatrix3d:
        """
        Создает матрицу ортогональной проекции на заданную плоскость.
        plane_origin: PyGePoint3d - точка на плоскости.
        plane_normal: PyGeVector3d - вектор нормали к плоскости.
        """
        
        # 1. Нормализуем вектор плоскости
        n_vec = np.array([plane_normal.x, plane_normal.y, plane_normal.z])
        n_vec = n_vec / np.linalg.norm(n_vec)
        Nx, Ny, Nz = n_vec

        # 2. Базовая матрица проекции (формула: I - N*N^T) (3x3)
        M_base_3x3 = np.array([
            [1 - Nx**2,   -Nx*Ny,    -Nx*Nz],
            [-Nx*Ny,    1 - Ny**2,   -Ny*Nz],
            [-Nx*Nz,    -Ny*Nz,    1 - Nz**2]
        ])

        # 3. Преобразуем в гомогенную матрицу 4x4
        M_proj_base = np.eye(4)
        M_proj_base[:3, :3] = M_base_3x3
        
        M_proj = PyGeMatrix3d(M_proj_base)

        # 4. Комбинация сдвигов, если плоскость не проходит через начало координат
        if plane_origin.x != 0 or plane_origin.y != 0 or plane_origin.z != 0:
            # T1: Сдвигаем начало координат на точку плоскости
            T1 = PyGeMatrix3d.translation(PyGeVector3d(-plane_origin.x, -plane_origin.y, -plane_origin.z))
            # T2: Обратный сдвиг
            T2 = PyGeMatrix3d.translation(PyGeVector3d(plane_origin.x, plane_origin.y, plane_origin.z))
            
            # M = T2 @ M_proj @ T1
            combined_matrix = T2.matrix @ M_proj.matrix @ T1.matrix
            return PyGeMatrix3d(combined_matrix)
        
        return M_proj

    def to_array(self): return np.array(self.matrix, dtype=np.float64)

    def __mul__(self, other: PyGeMatrix3d|int|float) -> PyGeMatrix3d:
        """Умножение матриц (Matrix *= Matrix и Matrix *= Scalar)"""
        if isinstance(other, PyGeMatrix3d):
            # Матричное умножение NumPy
            result_matrix = self.matrix @ other.matrix
            return PyGeMatrix3d(result_matrix)
        elif isinstance(other, (int, float)):
            result_matrix = self.matrix * other
            return PyGeMatrix3d(result_matrix)
        raise TypeError(f"Неподдерживаемый тип операнда {type(other)}")

    def __imul__(self, other: PyGeMatrix3d|int|float) -> PyGeMatrix3d:
        """Умножение матриц (Matrix *= Matrix и Matrix *= Scalar)"""
        if isinstance(other, PyGeMatrix3d):
            # Матричное умножение NumPy
            self.matrix = self.matrix @ other.matrix
            return self
        elif isinstance(other, (int, float)):
            self.matrix = self.matrix * other
            return self
        raise TypeError(f"Неподдерживаемый тип операнда {type(other)}")
    
    def invert(self):
        """Инвертирует текущую матрицу"""
        self.matrix = np.linalg.inv(self.matrix)
        return self

    def inverse(self):
        """Возвращает новую инвертированную матрицу"""
        inverted_matrix = np.linalg.inv(self.matrix)
        return PyGeMatrix3d(inverted_matrix)

    def transposeIt(self) -> PyGeMatrix3d:
        """Транспонирует текущую матрицу"""
        self.matrix = self.matrix.T
        return self

    def transpose(self) -> PyGeMatrix3d:
        """Возвращает транспонированную матрицу"""
        return PyGeMatrix3d(self.matrix.T)

    def __call__(self):
        """Возвращает VARIANT для методов CDispatch"""
        return Variant(self.matrix,VT.ARRAY_DOUBLE).to_variant()




class PyGeMatrix2d:
    """Матрица трансформации 3x3."""

    def __init__(self, matrix_data: PyGeMatrix3d|np.ndarray|None = None):
        # По умолчанию создаем единичную матрицу 3x3 (Identity Matrix)
        if matrix_data is None:
            self.matrix:np.ndarray = np.eye(3, dtype=float)
        elif isinstance(matrix_data, np.ndarray) and matrix_data.shape == (3, 3):
            self.matrix:np.ndarray = matrix_data
        elif isinstance(matrix_data, PyGeMatrix3d):
            self.matrix:np.ndarray = matrix_data.matrix.copy()
        else:
            raise ValueError("Матрица должна быть numpy-массивом 3x3 или PyGeMatrix2d")

    def __str__(self):
        return f"{self.__class__.__name__}(\n{self.matrix}\n)"

    
    @staticmethod
    def translation(vector: PyGeVector2d) -> PyGeMatrix2d:
        """Создает матрицу сдвига (трансляции) по вектору PyGeVector2d"""
        m = np.eye(3)
        m[0, 2] = vector.x
        m[1, 2] = vector.y
        return PyGeMatrix2d(m)

    @staticmethod
    def rotation(angle_rad: float, center: PyGePoint2d = None) -> PyGeMatrix2d:
        """
        Создает матрицу поворота на заданный угол (в радианах) вокруг точки.
        В 2D вращение всегда происходит вокруг оси Z.
        """
        cos_theta = math.cos(angle_rad)
        sin_theta = math.sin(angle_rad)
        
        # Базовая матрица поворота 3x3 относительно начала координат
        m_rotate_base = np.array([
            [cos_theta, -sin_theta, 0],
            [sin_theta, cos_theta,  0],
            [0,         0,          1]
        ])
        
        M_rotate = PyGeMatrix2d(m_rotate_base)

        if center is None:
            # Поворот относительно начала координат
            return M_rotate
        else:
            # Поворот относительно произвольного центра (комбинация сдвигов)
            # T2 * R * T1
            T1 = PyGeMatrix2d.translation(PyGeVector2d(-center.x, -center.y))
            T2 = PyGeMatrix2d.translation(PyGeVector2d(center.x, center.y))
            
            # Комбинируем матрицы: T2 @ M_rotate @ T1
            combined_matrix = T2.matrix @ M_rotate.matrix @ T1.matrix
            return PyGeMatrix2d(combined_matrix)

    @staticmethod
    def scaling(scale_factor: float, center: PyGePoint2d = None) -> PyGeMatrix2d:
        """Создает матрицу масштабирования. Если указан center, масштабирование происходит относительно этой точки."""

        # 1. Сначала создаем базовую матрицу масштабирования относительно начала координат
        m_scale_base = np.eye(4)
        m_scale_base[0, 0] = scale_factor
        m_scale_base[1, 1] = scale_factor
        
        # Оборачиваем ее в объект PyGeMatrix2d
        S = PyGeMatrix2d(m_scale_base)
        
        if center is None:
            # Если центр не указан, возвращаем S
            return S
        else:
            # Комбинируем матрицы сдвига для масштабирования относительно центра
            T1 = PyGeMatrix2d.translation(PyGeVector2d(-center.x, -center.y))
            T2 = PyGeMatrix2d.translation(PyGeVector2d(center.x, center.y))
            
            # Комбинируем матрицы: T2 @ S @ T1
            combined_matrix = T2.matrix @ S.matrix @ T1.matrix
            return PyGeMatrix2d(combined_matrix)


    @staticmethod
    def mirroring(line_origin: PyGePoint2d, line_normal: PyGeVector2d) -> PyGeMatrix2d:
        """
        Создает матрицу отражения относительно заданной прямой (линии) в 2D.
        line_origin: PyGePoint2d - точка на прямой.
        line_normal: PyGeVector2d - вектор нормали к прямой.
        """
        
        # 1. Нормализуем вектор на всякий случай
        n_vec = np.array([line_normal.x, line_normal.y])
        n_vec = n_vec / np.linalg.norm(n_vec)
        Nx, Ny = n_vec

        # 2. Базовая матрица отражения 2x2 (относительно линии через начало координат)
        M_base_2x2 = np.array([
            [1 - 2*Nx**2,   -2*Nx*Ny],
            [-2*Nx*Ny,      1 - 2*Ny**2]
        ])

        # Преобразуем в гомогенную матрицу 3x3
        M_mirror_base = np.eye(3)
        M_mirror_base[:2, :2] = M_base_2x2
        
        M_mirror = PyGeMatrix2d(M_mirror_base)

        # 3. Комбинация сдвигов, если линия не проходит через начало координат
        if line_origin.x != 0 or line_origin.y != 0:
            T1 = PyGeMatrix2d.translation(PyGeVector2d(-line_origin.x, -line_origin.y))
            T2 = PyGeMatrix2d.translation(PyGeVector2d(line_origin.x, line_origin.y))
            
            # M = T2 @ M_mirror @ T1
            combined_matrix = T2.matrix @ M_mirror.matrix @ T1.matrix
            return PyGeMatrix2d(combined_matrix)
            
        return M_mirror

    @staticmethod
    def projection(line_origin: PyGePoint2d, line_normal: PyGeVector2d) -> PyGeMatrix2d:
        """
        Создает матрицу ортогональной проекции на заданную прямую (линию) в 2D.
        line_origin: PyGePoint2d - точка на прямой.
        line_normal: PyGeVector2d - вектор нормали к прямой.
        """
        
        # 1. Нормализуем вектор прямой
        n_vec = np.array([line_normal.x, line_normal.y])
        n_vec = n_vec / np.linalg.norm(n_vec)
        Nx, Ny = n_vec

        # 2. Базовая матрица проекции (формула: I - N*N^T) (2x2)
        M_base_2x2 = np.array([
            [1 - Nx**2,   -Nx*Ny],
            [-Nx*Ny,    1 - Ny**2]
        ])

        # 3. Преобразуем в гомогенную матрицу 3x3
        M_proj_base = np.eye(3)
        M_proj_base[:2, :2] = M_base_2x2
        
        M_proj = PyGeMatrix2d(M_proj_base)

        # 4. Комбинация сдвигов, если прямая не проходит через начало координат
        if line_origin.x != 0 or line_origin.y != 0:
            T1 = PyGeMatrix2d.translation(PyGeVector2d(-line_origin.x, -line_origin.y))
            T2 = PyGeMatrix2d.translation(PyGeVector2d(line_origin.x, line_origin.y))
            
            # M = T2 @ M_proj @ T1
            combined_matrix = T2.matrix @ M_proj.matrix @ T1.matrix
            return PyGeMatrix2d(combined_matrix)
        
        return M_proj

    def __mul__(self, other: PyGeMatrix2d|int|float) -> PyGeMatrix2d:
        """Умножение матриц (Matrix *= Matrix и Matrix *= Scalar)"""
        if isinstance(other, PyGeMatrix2d):
            # Матричное умножение NumPy
            result_matrix = self.matrix @ other.matrix
            return PyGeMatrix2d(result_matrix)
        elif isinstance(other, (int, float)):
            result_matrix = self.matrix * other
            return PyGeMatrix2d(result_matrix)
        raise TypeError(f"Неподдерживаемый тип операнда {type(other)}")

    def __imul__(self, other: PyGeMatrix2d|int|float) -> PyGeMatrix2d:
        """Умножение матриц (Matrix *= Matrix и Matrix *= Scalar)"""
        if isinstance(other, PyGeMatrix2d):
            # Матричное умножение NumPy
            self.matrix = self.matrix @ other.matrix
            return self
        elif isinstance(other, (int, float)):
            self.matrix = self.matrix * other
            return self
        raise TypeError(f"Неподдерживаемый тип операнда {type(other)}")
    
    def invert(self) -> None:
        """Инвертирует текущую матрицу"""
        self.matrix = np.linalg.inv(self.matrix)
        # return self

    def inverse(self) -> PyGeMatrix2d:
        """Возвращает новую инвертированную матрицу"""
        inverted_matrix = np.linalg.inv(self.matrix)
        return PyGeMatrix2d(inverted_matrix)
    
    def transposeIt(self) -> None:
        """Транспонирует текущую матрицу"""
        self.matrix = self.matrix.T
        # return self

    def transpose(self) -> PyGeMatrix2d:
        """Возвращает транспонированную матрицу"""
        return PyGeMatrix2d(self.matrix.T)

    def __call__(self):
        """Возвращает Variant для методов CDispatch"""
        return Variant(self.matrix,VT.ARRAY_DOUBLE).to_variant()


from __future__ import annotations
from ..Base import *
from ..Proxy import *
from ..AcadEntity import AcadEntity



class AcadMtext(AcadEntity):
    def __init__(self, obj) -> None: super().__init__(obj)

    AttachmentPoint: AcAttachmentPoint = proxy_property('AcAttachmentPoint','AttachmentPoint',AccessMode.ReadWrite)
    BackgroundFill: bool = proxy_property(bool,'BackgroundFill',AccessMode.ReadWrite)
    DrawingDirection: AcDrawingDirection = proxy_property('AcDrawingDirection','DrawingDirection',AccessMode.ReadWrite)
    Height: float = proxy_property(float,'Height',AccessMode.ReadWrite)
    InsertionPoint: PyGePoint3d = proxy_property('PyGePoint3d','InsertionPoint',AccessMode.ReadWrite)
    LineSpacingDistance: float = proxy_property(float,'LineSpacingDistance',AccessMode.ReadWrite)
    LineSpacingFactor: float = proxy_property(float,'LineSpacingFactor',AccessMode.ReadWrite)
    LineSpacingStyle: AcLineSpacingStyle = proxy_property('AcLineSpacingStyle','LineSpacingStyle',AccessMode.ReadWrite)
    Normal: PyGeVector3d = proxy_property('PyGeVector3d','Normal',AccessMode.ReadWrite)
    TextString: str = proxy_property(str,'TextString',AccessMode.ReadWrite)
    Width: float = proxy_property(float,'TextString',AccessMode.ReadWrite)

    def FieldCode(self) -> str:
        return self._obj.FieldCode()

    def Copy(self) -> AcadMtext:
        return AcadMtext(self._obj.Copy())



from __future__ import annotations
from ..Base import *
from ..Proxy import *
from ..AcadEntity import AcadEntity



class AcadText(AcadEntity):
    def __init__(self, obj) -> None: super().__init__(obj)

    Alignment: AcAlignment = proxy_property('AcAlignment','Alignment',AccessMode.ReadWrite)
    Backward: bool = proxy_property(bool,'Backward',AccessMode.ReadWrite)
    Height: float = proxy_property(float,'Height',AccessMode.ReadWrite)
    InsertionPoint: PyGePoint3d = proxy_property('PyGePoint3d','InsertionPoint',AccessMode.ReadWrite)

    Normal: PyGeVector3d = proxy_property('PyGeVector3d','Normal',AccessMode.ReadWrite)
    ObliqueAngle: float = proxy_property(float,'ObliqueAngle',AccessMode.ReadWrite)
    TextAlignmentPoint: PyGePoint3d = proxy_property('PyGePoint3d','TextAlignmentPoint',AccessMode.ReadWrite)
    TextGenerationFlag: AcTextGenerationFlag = proxy_property('AcTextGenerationFlag','TextGenerationFlag',AccessMode.ReadWrite)
    TextString: str = proxy_property(str,'TextString',AccessMode.ReadWrite)
    Thickness: float = proxy_property(float,'Thickness',AccessMode.ReadWrite)
    UpsideDown: bool = proxy_property(bool,'UpsideDown',AccessMode.ReadWrite)

    def Copy(self) -> AcadText:
        return AcadText(self._obj.Copy())
    
    def FieldCode(self) -> str:
        return self._obj.FieldCode()



from __future__ import annotations
from ..Base import *
from ..Proxy import *
from ..AcadObject import *



class AcadLayer(AcadObject):
    def __init__(self, obj) -> None: super().__init__(obj)

    Application: AcadApplication = proxy_property('AcadApplication','Application', AccessMode.ReadOnly)
    Description: str = proxy_property(str,'Description',AccessMode.ReadWrite)
    Document: AcadDocument = proxy_property('AcadDocument','Document',AccessMode.ReadOnly)
    Freeze: bool = proxy_property(bool,'Freeze',AccessMode.ReadWrite)
    Handle: int = proxy_property(int,'Handle',AccessMode.ReadOnly)
    HasExtensionDictionary: bool = proxy_property(bool,'HasExtensionDictionary',AccessMode.ReadOnly)
    LayerOn: bool = proxy_property(bool,'LayerOn',AccessMode.ReadWrite)
    Linetype: str = proxy_property(str,'Linetype',AccessMode.ReadWrite)
    Lineweight: AcLineWeight = proxy_property('AcLineWeight','Lineweight',AccessMode.ReadWrite)


# class acDefaultLinetype(Enum):
#     Continuous = 'Continuous'
#     ByLayer = 'ByLayer'
#     ByBlock = 'ByBlock'



from __future__ import annotations
from ..Base import *
from ..Proxy import *
from ..AcadEntity import AcadEntity



class AcadPointCloud(AcadEntity):
    def __init__(self, obj) -> None: super().__init__(obj)

    Height: float = proxy_property(float,'Height',AccessMode.ReadWrite)
    InsertionPoint: PyGePoint3d = proxy_property('PyGePoint3d','InsertionPoint',AccessMode.ReadWrite)
    IntensityColorScheme: AcPointCloudIntensityStyle = proxy_property('AcPointCloudIntensityStyle','IntensityColorScheme',AccessMode.ReadWrite)
    Locked: bool = proxy_property(bool,'Locked',AccessMode.ReadWrite)
    Name: str = proxy_property(str,'Name',AccessMode.ReadWrite)
    Path: str = proxy_property(str,'Path',AccessMode.ReadOnly)
    Rotation: float = proxy_property(float,'Rotation',AccessMode.ReadWrite)
    Scale: float = proxy_property(float,'Scale',AccessMode.ReadWrite)
    ShowClipped: bool = proxy_property(bool,'ShowClipped',AccessMode.ReadWrite)
    ShowIntensity: bool = proxy_property(bool,'ShowIntensity',AccessMode.ReadWrite)
    Stylization: AcPointCloudStylizationType = proxy_property('AcPointCloudStylizationType','Stylization',AccessMode.ReadWrite)
    Unit: str = proxy_property(str,'Unit',AccessMode.ReadOnly)
    UnitFactor: str = proxy_property(str,'UnitFactor',AccessMode.ReadOnly)
    UseEntityColor: AcPointCloudColorType = proxy_property('AcPointCloudColorType','UseEntityColor',AccessMode.ReadWrite)
    Width: float = proxy_property(float,'Width',AccessMode.ReadWrite)

    def Copy(self) -> AcadPointCloud:
        return AcadPointCloud(self._obj.Copy())


from ..Base import *
from ..Proxy import *
from ..AcadEntity import AcadEntity as AcadEntity

class AcadHelix(AcadEntity):
    def __init__(self, obj) -> None: ...
    BaseRadius: float
    Constrain: AcHelixConstrainType
    Height: float
    Position: PyGePoint3d
    TopRadius: float
    TotalLength: float
    TurnHeight: float
    Turns: int
    TurnSlope: float
    Twist: AcHelixTwistType
    def Copy(self) -> AcadHelix: ...



from ..Base import *
from ..Proxy import *
from ..AcadEntity import AcadEntity as AcadEntity

class Acad3DFace(AcadEntity):
    def __init__(self, obj) -> None: ...
    Coordinates: PyGePoint3dArray
    VisibilityEdge1: bool
    VisibilityEdge2: bool
    VisibilityEdge3: bool
    VisibilityEdge4: bool
    def Coordinate(self, Index: int) -> PyGePoint3d: ...
    def Copy(self) -> Acad3DFace: ...
    def GetInvisibleEdge(self, Index: int) -> bool: ...
    def SetInvisibleEdge(self, Index: int, State: bool) -> None: ...



from __future__ import annotations
# from ..Base import *
from ..Proxy import proxy_property, AccessMode
from ..AcadObject import *

class AcadRegisteredApplication(AcadObject):
    def __init__(self, obj) -> None: super().__init__(obj)

    Name: str = proxy_property(str,'Name',AccessMode.ReadWrite)


from ..Base import *
from ..Proxy import *
from .AcadPopupMenu import *

class AcadPopupMenus(AppObject):
    def __init__(self, obj) -> None: ...
    Application: AcadApplication
    Count: int
    Parent: AppObject
    def Add(self, Name: str) -> AcadPopupMenu: ...
    def InsertMenuInMenuBar(self, MenuName: str, Index: int) -> None: ...
    def Item(self, Index: int) -> AcadPopupMenu: ...
    def RemoveMenuFromMenuBar(self, Index: int) -> None: ...
    def __iter__(self): ...



from ..Base import *
from ..Proxy import *
from ..AcadObject import *
from .AcadHyperlink import *

class AcadHyperlinks(AppObjectCollection):
    def __init__(self, obj) -> None: ...
    Application: AcadApplication
    Count: int
    def Add(self, Name: str, Description: str = '', NamedLocation: str = '') -> AcadHyperlink: ...
    def Item(self, Index: int | str) -> AcadHyperlink: ...
    def __iter__(self): ...



from __future__ import annotations
from ..Base import *
from ..Proxy import *
from ..AcadObject import *

class AcadToolbar(AppObject):
    def __init__(self, obj) -> None: super().__init__(obj)

    Application: AcadApplication = proxy_property('AcadApplication','Application',AccessMode.ReadOnly)
    Count: int = proxy_property(int,'Count',AccessMode.ReadOnly)
    DockStatus: AcToolbarDockStatus = proxy_property('AcToolbarDockStatus','DockStatus',AccessMode.ReadOnly)
    FloatingRows: int = proxy_property(int,'FloatingRows',AccessMode.ReadWrite)
    Height: int = proxy_property(int,'Height',AccessMode.ReadOnly)
    HelpString: str = proxy_property(str,'HelpString',AccessMode.ReadWrite)
    LargeButtons: bool = proxy_property(bool,'LargeButtons',AccessMode.ReadOnly)
    Left: int = proxy_property(int,'Left',AccessMode.ReadWrite)
    Name: str = proxy_property(str,'Name',AccessMode.ReadWrite)
    Parent: AppObject = proxy_property('AppObject','Parent',AccessMode.ReadWrite)
    TagString: str = proxy_property(str,'TagString',AccessMode.ReadOnly)
    Top: int = proxy_property(int,'Top',AccessMode.ReadWrite)
    Visible: bool = proxy_property(bool,'Visible',AccessMode.ReadWrite)
    Width: float = proxy_property(float,'Width',AccessMode.ReadOnly)

    def AddSeparator(self, Index: int | str) -> AcadToolbarItem:
        return AcadToolbarItem(self._obj.AddSeparator(Index))

    def AddToolbarButton(self, Index: int | str, Name: str, HelpString: str, Macro: str, FlyoutButton: vBool = None) -> AcadToolbarItem:
        if FlyoutButton is None:
            return AcadToolbarItem(self._obj.AddToolbarButton(Index, Name, HelpString, Macro))
        else:
            return AcadToolbarItem(self._obj.AddToolbarButton(Index, Name, HelpString, Macro, FlyoutButton()))

    def Delete(self) -> None:
        self._obj.Delete()

    def Dock(self, Side: AcToolbarDockStatus) -> None:
        self._obj.Dock(Side)

    def Float(self, Top: int, Left: int, NumberFloatRows: int) -> None:
        self._obj.Float(Top, Left, NumberFloatRows)

    def Item(self, Index: int | str) -> AcadObject:
        return AcadObject(self._obj.Item(Index))



from __future__ import annotations
from ..Base import *
from ..Proxy import *
from ..AcadEntity import AcadEntity



class AcadGeoPositionMarker(AcadEntity):
    def __init__(self, obj) -> None: super().__init__(obj)

    Altitude: float = proxy_property(float,'Altitude',AccessMode.ReadWrite)
    BackgroundFill: bool = proxy_property(bool,'BackgroundFill',AccessMode.ReadWrite)
    DrawingDirection: AcDrawingDirection = proxy_property('AcDrawingDirection','DrawingDirection',AccessMode.ReadWrite)
    Height: float = proxy_property(float,'Height',AccessMode.ReadWrite)
    LandingGap: float = proxy_property(float,'LandingGap',AccessMode.ReadWrite)
    Latitude: str = proxy_property(str,'Latitude',AccessMode.ReadWrite)
    LineSpacingDistance: float = proxy_property(float,'LineSpacingDistance',AccessMode.ReadWrite)
    LineSpacingFactor: float = proxy_property(float,'LineSpacingFactor',AccessMode.ReadWrite)
    LineSpacingStyle: AcLineSpacingStyle = proxy_property('AcLineSpacingStyle','LineSpacingStyle',AccessMode.ReadWrite)
    Longitude: str = proxy_property(str,'Longitude',AccessMode.ReadWrite)
    Notes: str = proxy_property(str,'Notes',AccessMode.ReadWrite)
    Position: PyGePoint3d = proxy_property('PyGePoint3d','Position',AccessMode.ReadWrite)
    Radius: float = proxy_property(float,'Radius',AccessMode.ReadWrite)
    Rotation: float = proxy_property(float,'Rotation',AccessMode.ReadWrite)
    TextFrameDisplay: bool = proxy_property(bool,'TextFrameDisplay',AccessMode.ReadWrite)
    TextJustify: AcAttachmentPoint = proxy_property('AcAttachmentPoint','TextJustify',AccessMode.ReadWrite)
    TextString: str = proxy_property(str,'TextString',AccessMode.ReadWrite)
    TextStyleName: str = proxy_property(str,'TextStyleName',AccessMode.ReadWrite)
    TextWidth: float = proxy_property(float,'TextWidth',AccessMode.ReadWrite)

    def Copy(self) -> AcadGeoPositionMarker:
        return AcadGeoPositionMarker(self._obj.Copy())


from ..AcadObject import *
from .AcadRegisteredApplication import AcadRegisteredApplication as AcadRegisteredApplication

class AcadRegisteredApplications(IAcadObjectCollection):
    def __init__(self, obj) -> None: ...
    def Add(self, Name: str) -> AcadRegisteredApplication: ...
    def Item(self, Index: int) -> AcadRegisteredApplication: ...
    def __iter__(self): ...



from __future__ import annotations
from ..Base import *
from ..Proxy import proxy_property, AccessMode
from ..AcadObject import *

class AcadPreferences(AppObject):
    def __init__(self, obj) -> None: super().__init__(obj)

    Application: AcadApplication = proxy_property('AcadApplication','Application',AccessMode.ReadOnly)
    Display: AcadPreferencesDisplay = proxy_property('AcadPreferencesDisplay','Display',AccessMode.ReadOnly)
    Drafting: AcadPreferencesDrafting = proxy_property('AcadPreferencesDrafting','Drafting',AccessMode.ReadOnly)
    Files: AcadPreferencesFiles = proxy_property('AcadPreferencesFiles','Files',AccessMode.ReadOnly)
    OpenSave: AcadPreferencesOpenSave = proxy_property('AcadPreferencesOpenSave','OpenSave',AccessMode.ReadOnly)
    Output: AcadPreferencesOutput = proxy_property('AcadPreferencesOutput','Output',AccessMode.ReadOnly)
    Profiles: AcadPreferencesProfiles = proxy_property('AcadPreferencesProfiles','Profiles',AccessMode.ReadOnly)
    Selection: AcadPreferencesSelection = proxy_property('AcadPreferencesSelection','Selection',AccessMode.ReadOnly)
    System: AcadPreferencesSystem = proxy_property('AcadPreferencesSystem','System',AccessMode.ReadOnly)
    User: AcadPreferencesUser = proxy_property('AcadPreferencesUser','User',AccessMode.ReadOnly)


from __future__ import annotations
from ..Base import *
from ..Proxy import *
from .AcadBlockReference import AcadBlockReference



class AcadMInsertBlock(AcadBlockReference):
    def __init__(self, obj) -> None: super().__init__(obj)

    Columns: int = proxy_property(int,'Columns',AccessMode.ReadWrite)
    ColumnSpacing: float = proxy_property(float,'ColumnSpacing',AccessMode.ReadWrite)

    def Copy(self) -> AcadMInsertBlock:
        return AcadMInsertBlock(self._obj.Copy())


from ..Base import *
from ..Proxy import *
from .AcadSubEntity import *

class AcadSubDMeshFace(AcadSubEntity):
    def __init__(self, obj) -> None: ...
    CreaseLevel: float
    CreaseType: AcMeshCreaseType
    Material: str



from __future__ import annotations
from ..Base import *
from ..Proxy import proxy_property, AccessMode
from ..AcadObject import *



class AcadMenuGroup(AppObject):
    def __init__(self, obj) -> None: super().__init__(obj)

    Application: 'AcadApplication' = proxy_property('AcadApplication','Application',AccessMode.ReadOnly)
    MenuFileName: str = proxy_property(str,'MenuFileName',AccessMode.ReadOnly)
    Menus: AcadPopupMenus = proxy_property('AcadPopupMenus','Menus',AccessMode.ReadOnly)
    Name: str = proxy_property(str,'Name',AccessMode.ReadWrite)
    Parent: AppObject = proxy_property('AppObject','Parent',AccessMode.ReadWrite)
    Toolbars: AcadToolbars = proxy_property('AcadToolbars','Toolbars',AccessMode.ReadOnly)
    Type: AcMenuGroupType  = proxy_property('AcMenuGroupType','Type',AccessMode.ReadOnly)

    def Save(self, MenuFileType: AcMenuFileType) -> None: 
        self._obj.Save(MenuFileType)

    def SaveAs(self, MenuFileName: str, MenuFileType: AcMenuFileType) -> None: 
        self._obj.SaveAs(MenuFileName, MenuFileType)
        
    def Unload(self) -> None: 
        self._obj.Unload()


from ..Base import *
from ..Proxy import *
from ..AcadEntity import AcadEntity as AcadEntity

class AcadPolygonMesh(AcadEntity):
    def __init__(self, obj) -> None: ...
    Coordinates: PyGePoint3dArray
    MClose: bool
    MDensity: int
    MVertexCount: int
    NClose: bool
    NDensity: int
    NVertexCount: int
    Type: AcPolymeshType
    def Coordinate(self, Index: int) -> PyGePoint3d: ...
    def Copy(self) -> AcadPolygonMesh: ...
    def AppendVertex(self, Point: PyGePoint3d) -> None: ...
    def Explode(self) -> vObjectArray: ...



from ..Base import *
from ..Proxy import *
from ..AcadEntity import AcadEntity as AcadEntity

class AcadTolerance(AcadEntity):
    def __init__(self, obj) -> None: ...
    DimensionLineColor: AcColor
    DirectionVector: PyGeVector3d
    InsertionPoint: PyGePoint3d
    Normal: PyGeVector3d
    ScaleFactor: float
    StyleName: str
    TextColor: AcColor
    TextHeight: float
    TextString: str
    TextStyle: AcadTextStyle



from ..Base import *
from ..Proxy import *

class AcadDynamicBlockReferenceProperty(AppObject):
    def __init__(self, obj) -> None: ...
    AllowedValues: any
    Description: str
    PropertyName: str
    ReadOnly: bool
    Show: bool
    UnitsType: AcDynamicBlockReferencePropertyUnitsType
    Value: any



from ..Base import *
from ..Proxy import *
from ..AcadEntity import AcadEntity as AcadEntity

class AcadText(AcadEntity):
    def __init__(self, obj) -> None: ...
    Alignment: AcAlignment
    Backward: bool
    Height: float
    InsertionPoint: PyGePoint3d
    Normal: PyGeVector3d
    ObliqueAngle: float
    TextAlignmentPoint: PyGePoint3d
    TextGenerationFlag: AcTextGenerationFlag
    TextString: str
    Thickness: float
    UpsideDown: bool
    def Copy(self) -> AcadText: ...
    def FieldCode(self) -> str: ...



from __future__ import annotations
from ..Base import *
from ..Proxy import *
from ..AcadEntity import AcadEntity



class AcadHelix(AcadEntity):
    def __init__(self, obj) -> None: super().__init__(obj)

    BaseRadius: float = proxy_property(float,'BaseRadius',AccessMode.ReadWrite)
    Constrain: AcHelixConstrainType = proxy_property('AcHelixConstrainType','Constrain',AccessMode.ReadWrite)
    Height: float = proxy_property(float,'Height',AccessMode.ReadWrite)
    Position: PyGePoint3d = proxy_property('PyGePoint3d','Position',AccessMode.ReadWrite)
    TopRadius: float = proxy_property(float,'TopRadius',AccessMode.ReadWrite)
    TotalLength: float = proxy_property(float,'TotalLength',AccessMode.ReadWrite)
    TurnHeight: float = proxy_property(float,'TurnHeight',AccessMode.ReadWrite)
    Turns: int = proxy_property(int,'Turns',AccessMode.ReadWrite)
    TurnSlope: float = proxy_property(float,'TurnSlope',AccessMode.ReadWrite)
    Twist: AcHelixTwistType = proxy_property('AcHelixTwistType','Twist',AccessMode.ReadWrite)

    def Copy(self) -> AcadHelix:
        return AcadHelix(self._obj.Copy())


from ..Base import *
from ..Proxy import *
from ..AcadObject import *
from .AcadUCS import AcadUCS as AcadUCS

class AcadUCSs(IAcadObjectCollection):
    def __init__(self, obj) -> None: ...
    def Add(self, Origin: PyGePoint3d, XAxisPoint: PyGePoint3d, YAxisPoint: PyGePoint3d, Name: str) -> AcadUCS: ...
    def Item(self, Index: int) -> AcadUCS: ...
    def __iter__(self): ...



from ..VarType import *

class PyGeVector3d(vDoubleArray):
    def __init__(self, *args) -> None: ...
    @property
    def x(self): ...
    @x.setter
    def x(self, value) -> None: ...
    @property
    def y(self): ...
    @y.setter
    def y(self, value) -> None: ...
    @property
    def z(self): ...
    @z.setter
    def z(self, value) -> None: ...
    def __add__(self, vector: PyGeVector3d) -> PyGeVector3d: ...
    def __sub__(self, vector: PyGeVector3d) -> PyGeVector3d: ...
    def __mul__(self, scalar: int | float) -> PyGeVector3d: ...
    def __rmul__(self, scalar: int | float) -> PyGeVector3d: ...
    def __truediv__(self, scalar: int | float) -> PyGeVector3d: ...
    def __neg__(self) -> PyGeVector3d: ...
    def __iadd__(self, vector) -> PyGeVector3d: ...
    def __isub__(self, vector) -> PyGeVector3d: ...
    def __imul__(self, scalar) -> PyGeVector3d: ...
    def __itruediv__(self, scalar) -> PyGeVector3d: ...
    def length(self) -> float: ...
    def lengthSqrd(self) -> float: ...
    def normalize(self) -> None: ...
    def normal(self) -> PyGeVector3d: ...
    def dotProduct(self, vector: PyGeVector3d) -> float: ...
    def crossProduct(self, vector: PyGeVector3d) -> PyGeVector3d: ...
    def angleTo(self, vector: PyGeVector3d) -> float: ...
    def isEqualTo(self, vector: PyGeVector3d, tolerance: float = 1e-09) -> bool: ...
    def negate(self) -> None: ...
    kIdentity: PyGeVector3d
    kXaxis: PyGeVector3d
    kYaxis: PyGeVector3d
    kZaxis: PyGeVector3d

class PyGeVector2d(vDoubleArray):
    def __init__(self, *args) -> None: ...
    @property
    def x(self): ...
    @x.setter
    def x(self, value) -> None: ...
    @property
    def y(self): ...
    @y.setter
    def y(self, value) -> None: ...
    def __add__(self, vector: PyGeVector2d) -> PyGeVector3d: ...
    def __sub__(self, vector: PyGeVector2d) -> PyGeVector2d: ...
    def __mul__(self, scalar: int | float) -> PyGeVector2d: ...
    def __rmul__(self, scalar: int | float) -> PyGeVector2d: ...
    def __truediv__(self, scalar: int | float) -> PyGeVector2d: ...
    def __neg__(self) -> PyGeVector2d: ...
    def __iadd__(self, vector) -> PyGeVector2d: ...
    def __isub__(self, vector) -> PyGeVector2d: ...
    def __imul__(self, scalar) -> PyGeVector2d: ...
    def __itruediv__(self, scalar) -> PyGeVector2d: ...
    def length(self) -> float: ...
    def lengthSqrd(self) -> float: ...
    def normalize(self) -> PyGeVector2d: ...
    def normal(self) -> PyGeVector2d: ...
    def dotProduct(self, vector: PyGeVector2d) -> float: ...
    def crossProduct(self, vector: PyGeVector2d) -> float: ...
    def angleTo(self, vector: PyGeVector2d) -> float: ...
    def isEqualTo(self, vector: PyGeVector2d, tolerance: float = 1e-09) -> bool: ...
    def negate(self) -> PyGeVector2d: ...
    kIdentity: PyGeVector2d
    kXaxis: PyGeVector2d
    kYaxis: PyGeVector2d



from __future__ import annotations
from ..Base import *
from ..Proxy import proxy_property, AccessMode
from ..AcadObject import *



class AcadGroup(IAcadObjectCollection):
    def __init__(self, obj) -> None: super().__init__(obj)

    Layer: str = proxy_property(str,'Layer',AccessMode.WriteOnly) #Group object which is write-only
    Linetype: str = proxy_property(str,'Linetype',AccessMode.WriteOnly) #Group object which is write-only
    LinetypeScale: float = proxy_property(float,'LinetypeScale',AccessMode.WriteOnly) #Group object which is write-only
    Lineweight: AcLineWeight = proxy_property('AcLineWeight','Lineweight',AccessMode.ReadWrite)
    Material: str = proxy_property(str,'Material',AccessMode.ReadWrite)
    Name: str = proxy_property(str,'Name',AccessMode.ReadWrite)
    PlotStyleName: str = proxy_property(str,'PlotStyleName',AccessMode.WriteOnly) #Group object which is write-only
    TrueColor: AcadAcCmColor = proxy_property('AcadAcCmColor','TrueColor',AccessMode.WriteOnly) #Group object which is write-only
    Visible: bool = proxy_property(bool,'Visible',AccessMode.WriteOnly) #Group object which is write-only

    def AppendItems(self, Objects: vObjectArray) -> None: 
        self._obj.AppendItems(Objects())

    def Highlight(self, HighlightFlag: bool) -> None: 
        self._obj.Highlight(HighlightFlag)

    def RemoveItems(self, Objects: vObjectArray) -> None:
        self._obj.RemoveItems(Objects())

    def Update(self) -> None:
        self._obj.Update()



from ..Base import *
from ..AcadObject import *
from .AcadSubEntity import *
from ..Proxy import AccessMode as AccessMode, proxy_property as proxy_property

class AcadMLeaderLeader(AcadSubEntity):
    def __init__(self, obj) -> None: ...
    ArrowheadBlock: str
    ArrowheadSize: int
    ArrowheadType: AcDimArrowheadType
    LeaderLineColor: AcadAcCmColor
    LeaderLinetype: str
    LeaderLineWeight: AcLineWeight
    LeaderType: AcMLeaderType



from __future__ import annotations
from ..Base import *
from ..Proxy import proxy_property, AccessMode
from ..AcadObject import *



class AcadPreferencesProfiles(AppObject):
    def __init__(self, obj) -> None: super().__init__(obj)

    ActiveProfile: str = proxy_property(str,'ActiveProfile',AccessMode.ReadOnly)
    Application: AcadApplication = proxy_property('AcadApplication','Application',AccessMode.ReadOnly)

    def CopyProfile(self, oldProfileName: str, newProfileName: str) -> None:
        self._obj.CopyProfile(oldProfileName, newProfileName)

    def DeleteProfile(self, ProfileName: str) -> None:
        self._obj.DeleteProfile(ProfileName)
        
    def ExportProfile(self, Profile: str, RegFile: str) -> None:
        self._obj.ExportProfile(Profile, RegFile)

    def GetAllProfileNames(self) -> tuple[str]:
        pNames = self._obj.GetAllProfileNames()
        return pNames

    def ImportProfile(self, Profile: str, RegFile: str, IncludePathInfo: bool) -> None:
        self._obj.ImportProfile(Profile, RegFile, IncludePathInfo)

    def RenameProfile(self, origProfileName: str, newProfileName: str) -> None:
        self._obj.ImportProfile(origProfileName, newProfileName)
        
    def ResetProfile(self, Profile: str) -> None:
        self._obj.ResetProfile(Profile)


from .AcadAcCmColor import *
from .AcadApplication import *
from .AcadBlock import *
from .AcadBlocks import *
from .AcadDatabase import *
from .AcadDatabasePreferences import *
from .AcadDictionaries import *
from .AcadDictionary import *
from .AcadDimStyle import *
from .AcadDimStyles import *
from .AcadDocument import *
from .AcadDocuments import *
from .AcadDynamicBlockReferenceProperty import *
from .AcadGroup import *
from .AcadGroups import *
from .AcadHyperlink import *
from .AcadHyperlinks import *
from .AcadIDPair import *
from .AcadLayer import *
from .AcadLayers import *
from .AcadLayerStateManager import *
from .AcadLayout import *
from .AcadLayouts import *
from .AcadLineType import *
from .AcadLineTypes import *
from .AcadMaterial import *
from .AcadMaterials import *
from .AcadMenuBar import *
from .AcadMenuGroup import *
from .AcadMenuGroups import *
from .AcadMLeaderLeader import *
from .AcadMLeaderStyle import *
from .AcadModelSpace import *
from .AcadPaperSpace import *
from .AcadPlot import *
from .AcadPlotConfiguration import *
from .AcadPlotConfigurations import *
from .AcadPopupMenu import *
from .AcadPopupMenuItem import *
from .AcadPopupMenus import *
from .AcadPreferences import *
from .AcadPreferencesDisplay import *
from .AcadPreferencesDrafting import *
from .AcadPreferencesFiles import *
from .AcadPreferencesOpenSave import *
from .AcadPreferencesOutput import *
from .AcadPreferencesProfiles import *
from .AcadPreferencesSelection import *
from .AcadPreferencesSystem import *
from .AcadPreferencesUser import *
from .AcadRegisteredApplication import *
from .AcadRegisteredApplications import *
from .AcadSectionManager import *
from .AcadSectionSettings import *
from .AcadSectionTypeSettings import *
from .AcadSecurityParams import *
from .AcadSelectionSet import *
from .AcadSelectionSets import *
from .AcadSortentsTable import *
from .AcadState import *
from .AcadSubDMeshEdge import *
from .AcadSubDMeshFace import *
from .AcadSubDMeshVertex import *
from .AcadSubEntity import *
from .AcadSubEntSolidEdge import *
from .AcadSubEntSolidFace import *
from .AcadSubEntSolidNode import *
from .AcadSubEntSolidVertex import *
from .AcadSummaryInfo import *
from .AcadTableStyle import *
from .AcadTextStyle import *
from .AcadTextStyles import *
from .AcadToolbar import *
from .AcadToolbarItem import *
from .AcadToolbars import *
from .AcadUCS import *
from .AcadUCSs import *
from .AcadUtility import *
from .AcadView import *
from .AcadViewport import *
from .AcadViewports import *
from .AcadViews import *
from .AcadXRecord import *

__all__ = [
'AcadAcCmColor',
'AcadApplication',
'AcadBlock',
'AcadBlocks',
'AcadDatabase',
'IAcadDatabase',
'AcadDatabasePreferences',
'AcadDictionaries',
'AcadDictionary',
'AcadDimStyle',
'AcadDimStyles',
'AcadDocument',
'AcadDocuments',
'AcadDynamicBlockReferenceProperty',
'AcadGroup',
'AcadGroups',
'AcadHyperlink',
'AcadHyperlinks',
'AcadIDPair',
'AcadLayer',
'AcadLayers',
'AcadLayerStateManager',
'AcadLayout',
'AcadLayouts',
'AcadLineType',
'AcadLineTypes',
'AcadMaterial',
'AcadMaterials',
'AcadMenuBar',
'AcadMenuGroup',
'AcadMenuGroups',
'AcadMLeaderLeader',
'AcadMLeaderStyle',
'AcadModelSpace',
'AcadPaperSpace',
'AcadPlot',
'AcadPlotConfiguration',
'AcadPlotConfigurations',
'AcadPopupMenu',
'AcadPopupMenuItem',
'AcadPopupMenus',
'AcadPreferences',
'AcadPreferencesDisplay',
'AcadPreferencesDrafting',
'AcadPreferencesFiles',
'AcadPreferencesOpenSave',
'AcadPreferencesOutput',
'AcadPreferencesProfiles',
'AcadPreferencesSelection',
'AcadPreferencesSystem',
'AcadPreferencesUser',
'AcadRegisteredApplication',
'AcadRegisteredApplications',
'AcadSectionManager',
'AcadSectionSettings',
'AcadSectionTypeSettings',
'AcadSecurityParams',
'AcadSelectionSet',
'AcadSelectionSets',
'AcadSortentsTable',
'AcadState',
'AcadSubDMeshEdge',
'AcadSubDMeshFace',
'AcadSubDMeshVertex',
'AcadSubEntity',
'AcadSubEntSolidEdge',
'AcadSubEntSolidFace',
'AcadSubEntSolidNode',
'AcadSubEntSolidVertex',
'AcadSummaryInfo',
'AcadTableStyle',
'AcadTextStyle',
'AcadTextStyles',
'AcadToolbar',
'AcadToolbarItem',
'AcadToolbars',
'AcadUCS',
'AcadUCSs',
'AcadUtility',
'AcadView',
'AcadViewport',
'AcadViewports',
'AcadViews',
'AcadXRecord',
]


from __future__ import annotations
from ..Base import *
from ..Proxy import *
from ..AcadEntity import AcadEntity
from abc import ABC, abstractmethod



class AcadUnderlay(AcadEntity, ABC):
    def __init__(self, obj) -> None: super().__init__(obj)

    AdjustForBackground: bool = proxy_property(bool,'AdjustForBackground',AccessMode.ReadWrite)
    ClippingEnabled: bool = proxy_property(bool,'ClippingEnabled',AccessMode.ReadWrite)
    Contrast: int = proxy_property(int,'Contrast',AccessMode.ReadWrite)
    Fade: int = proxy_property(int,'Fade',AccessMode.ReadWrite)
    File: str = proxy_property(str,'File',AccessMode.ReadWrite)
    Height: float = proxy_property(float,'Height',AccessMode.ReadWrite)
    ItemName: str = proxy_property(str,'ItemName',AccessMode.ReadWrite)
    Monochrome: bool = proxy_property(bool,'Monochrome',AccessMode.ReadWrite)
    Position: PyGePoint3d = proxy_property('PyGePoint3d','Position',AccessMode.ReadWrite)
    Rotation: float = proxy_property(float,'Rotation',AccessMode.ReadWrite)
    ScaleFactor: float = proxy_property(float,'ScaleFactor',AccessMode.ReadWrite)
    UnderlayLayerOverrideApplied: bool = proxy_property(bool,'UnderlayLayerOverrideApplied',AccessMode.ReadWrite)
    UnderlayName: str = proxy_property(str,'UnderlayName',AccessMode.ReadWrite)
    UnderlayVisibility: bool = proxy_property(bool,'UnderlayVisibility',AccessMode.ReadWrite)
    Width: float = proxy_property(float,'Width',AccessMode.ReadWrite)

    def ClipBoundary(self, PointsArray: PyGePoint3dArray) -> None:
        self._obj.ClipBoundary(PointsArray)
    
    @abstractmethod
    def Copy(self): return self._obj.Copy()



from .Points import *
from .Matrix import *
from .Vector import *

__all__ = [
    'PyGePoint3d',
    'PyGePoint3dArray',
    'PyGePoint2d',
    'PyGePoint2dArray',
    'PyGeVector3d',
    'PyGeVector2d',
    'PyGeMatrix3d',
    'PyGeMatrix2d'
]


from ..Base import *
from ..Proxy import *
from .AcadSurface import AcadSurface as AcadSurface

class AcadSweptSurface(AcadSurface):
    def __init__(self, obj) -> None: ...
    Bank: bool
    ProfileRotation: float
    Scale: float
    Twist: float
    def Copy(self) -> AcadSweptSurface: ...



from __future__ import annotations
from ..Base import *
from ..Proxy import *
from ..AcadEntity import AcadEntity



class AcadTrace(AcadEntity):
    def __init__(self, obj) -> None: super().__init__(obj)

    Coordinates: PyGePoint3dArray = proxy_property('PyGePoint3dArray','Coordinates',AccessMode.ReadWrite)
    Normal: PyGeVector3d = proxy_property('PyGeVector3d','Normal',AccessMode.ReadWrite)
    Thickness: float = proxy_property(float,'Thickness',AccessMode.ReadWrite)

    def Coordinate(self, Index: int) -> PyGePoint3d:
        return PyGePoint3d(self._obj.Coordinate(Index))

    def Copy(self) -> AcadTrace:
        return AcadTrace(self._obj.Copy())


from __future__ import annotations
from ..Base import *
from ..Proxy import *
from ..AcadObject import *

class AcadTableStyle(AcadObject):
    def __init__(self, obj) -> None: super().__init__(obj)
    
    BitFlags: int = proxy_property(int,'BitFlags',AccessMode.ReadWrite)
    Description: str = proxy_property(str,'Description',AccessMode.ReadWrite)
    FlowDirection: AcTableDirection = proxy_property('AcTableDirection','FlowDirection',AccessMode.ReadWrite)
    HeaderSuppressed: bool = proxy_property(bool,'HeaderSuppressed',AccessMode.ReadWrite)
    HorzCellMargin: float = proxy_property(float,'HorzCellMargin',AccessMode.ReadWrite)
    Name: str = proxy_property(str,'Name',AccessMode.ReadWrite)
    NumCellStyles: int = proxy_property(int,'NumCellStyles',AccessMode.ReadOnly)
    TemplateId: int = proxy_property(int,'TemplateId',AccessMode.ReadWrite)
    TitleSuppressed: bool = proxy_property(bool,'TitleSuppressed',AccessMode.ReadWrite)
    VertCellMargin: float = proxy_property(float,'VertCellMargin',AccessMode.ReadWrite)

    def CreateCellStyle(self, StringCellStyle: str) -> None:
        self._obj.CreateCellStyle(StringCellStyle)

    def CreateCellStyleFromStyle(self, StringCellStyle: str, StringSourceCellStyle: str) -> None:
        self._obj.CreateCellStyleFromStyle(StringCellStyle, StringSourceCellStyle)

    def DeleteCellStyle(self, StringCellStyle: str) -> None:
        self._obj.DeleteCellStyle(StringCellStyle)

    def EnableMergeAll(self, nRow: int, nCol: int, bEnable: bool) -> None:
        self._obj.EnableMergeAll(nRow, nCol, bEnable)

    def GetAlignment(self, rowType: AcRowType) -> AcCellAlignment:
        return AcCellAlignment(self._obj.GetAlignment(rowType))

    def GetAlignment2(self, bstrCellStyle: str) -> AcCellAlignment:
        return AcCellAlignment(self._obj.GetAlignment2(bstrCellStyle))

    def GetBackgroundColor(self, rowType: AcRowType) -> AcadAcCmColor:
        return AcadAcCmColor(self._obj.GetBackgroundColor(rowType))

    def GetBackgroundColor2(self, bstrCellStyle: str) -> AcadAcCmColor:
        return AcadAcCmColor(self._obj.GetBackgroundColor2(bstrCellStyle))

    def GetBackgroundColorNone(self, rowType: AcRowType) -> bool:
        return self._obj.GetBackgroundColorNone(rowType)

    def GetCellClass(self, StringCellStyle: str) -> int:
        return self._obj.GetCellClass(StringCellStyle)

    def GetCellStyles(self) -> Variant:
        cellStylesArray = self._obj.GetCellStyles()
        return Variant(cellStylesArray)

    def GetColor(self, rowType: AcRowType) -> AcadAcCmColor:
        return AcadAcCmColor(self._obj.GetColor(rowType))

    def GetColor2(self, bstrCellStyle: str) -> AcadAcCmColor:
        return AcadAcCmColor(self._obj.GetColor2(bstrCellStyle))

    def GetDataType(self, rowType: AcRowType) -> tuple:
        pDataType, pUnitType = self._obj.GetDataType(rowType)
        return AcValueDataType(pDataType), AcValueUnitType(pUnitType)

    def GetDataType2(self, nRow: int, nCol: int, nContent: int) -> tuple:
        pDataType, pUnitType = self._obj.GetDataType2(nRow, nCol, nContent)
        return AcValueDataType(pDataType), AcValueUnitType(pUnitType)

    def GetFormat(self, rowType: AcRowType) -> str:
        return self._obj.GetFormat(rowType)

    def GetFormat2(self, StringCellStyle: str) -> str:
        pbstrFormat = self._obj.GetFormat2(StringCellStyle)
        return pbstrFormat

    def GetGridColor(self, gridLineType: AcGridLineType, rowType: AcRowType) -> AcadAcCmColor:
        return AcadAcCmColor(self._obj.GetGridColor(gridLineType, rowType))

    def GetGridColor2(self, bstrCellStyle: str, gridLineType: AcGridLineType) -> AcadAcCmColor:
        return AcadAcCmColor(self._obj.GetGridColor2(bstrCellStyle, gridLineType))

    def GetGridLineWeight(self, gridLineType: AcGridLineType, rowType: AcRowType) -> AcLineWeight:
        return AcLineWeight(self._obj.GetGridLineWeight(gridLineType, rowType))

    def GetGridLineWeight2(self, nRow: int, nCol: int, nGridLineType: AcGridLineType) -> AcLineWeight:
        return AcLineWeight(self._obj.GetGridLineWeight2(nRow, nCol, nGridLineType))

    def GetGridVisibility(self, gridLineType: AcGridLineType, rowType: AcRowType) -> bool:
        return self._obj.GetGridVisibility(gridLineType, rowType)

    def GetGridVisibility2(self, nRow: int, nCol: int, nGridLineType: AcGridLineType) -> bool:
        return self._obj.GetGridVisibility2(nRow, nCol, nGridLineType)

    def GetIsCellStyleInUse(self, pszCellStyle: str) -> bool:
        return self._obj.GetIsCellStyleInUse(pszCellStyle)

    def GetIsMergeAllEnabled(self, StringCellStyle: str) -> bool:
        return self._obj.GetIsMergeAllEnabled(StringCellStyle)

    def GetRotation(self, StringCellStyle: str) -> float:
        return self._obj.GetRotation(StringCellStyle)

    def GetTextHeight(self, rowType: AcRowType) -> float:
        return self._obj.GetTextHeight(rowType)

    def GetTextHeight2(self, StringCellStyle: str) -> float:
        return self._obj.GetTextHeight2(StringCellStyle)

    def GetTextStyle(self, rowType: AcRowType) -> str:
        return self._obj.GetTextStyle(rowType)

    def GetTextStyleId(self, bstrCellStyle: str) -> float:
        return self._obj.GetTextStyleId(bstrCellStyle)

    def GetUniqueCellStyleName(self, pszBaseName: str) -> str:
        return self._obj.GetUniqueCellStyleName(pszBaseName)

    def RenameCellStyle(self, StringOldName: str, StringNewName: str) -> None:
        self._obj.RenameCellStyle(StringOldName, StringNewName)

    def SetAlignment(self, rowTypes: AcRowType, cellAlignment: AcCellAlignment) -> None:
        self._obj.SetAlignment(rowTypes, cellAlignment)

    def SetAlignment2(self, bstrCellStyle: str, cellAlignment: AcCellAlignment) -> None:
        self._obj.SetAlignment2(bstrCellStyle, cellAlignment)

    def SetBackgroundColor(self, rowTypes: AcRowType, pColor: AcadAcCmColor) -> None:
        self._obj.SetBackgroundColor(rowTypes, pColor())

    def SetBackgroundColor2(self, bstrCellStyle: str, color: AcadAcCmColor) -> None:
        self._obj.SetBackgroundColor2(bstrCellStyle, color())

    def SetBackgroundColorNone(self, rowTypes: AcRowType, bValue: bool) -> None:
        self._obj.SetBackgroundColorNone(rowTypes, bValue)

    def SetCellClass(self, StringCellStyle: str, cellClass: int) -> None:
        self._obj.SetCellClass(StringCellStyle, cellClass)

    def SetColor(self, rowTypes: AcRowType, pColor: AcadAcCmColor) -> None:
        self._obj.SetColor(rowTypes, pColor())

    def SetColor2(self, bstrCellStyle: str, color: AcadAcCmColor) -> None:
        self._obj.SetColor2(bstrCellStyle, color())

    def SetDataType(self, rowTypes: AcRowType, nDataType: AcValueDataType, nUnitType: AcValueUnitType) -> None:
        self._obj.SetDataType(rowTypes, nDataType, nUnitType)

    def SetDataType2(self, nRow: int, nCol: int, nContent: int, dataType: AcValueDataType, unitType: AcValueUnitType) -> None:
        self._obj.SetDataType2(nRow, nCol, nContent, dataType, unitType)

    def SetFormat(self, rowTypes: AcRowType, pFormat: str) -> None:
        self._obj.SetFormat(rowTypes, pFormat)

    def SetFormat2(self, bstrCellStyle: str, bstrFormat: str) -> None:
        self._obj.SetFormat2(bstrCellStyle, bstrFormat)

    def SetGridColor(self, gridLineTypes: AcGridLineType, rowTypes: AcRowType, pColor: AcadAcCmColor) -> None:
        self._obj.SetGridColor(gridLineTypes, rowTypes, pColor())

    def SetGridColor2(self, nRow: int, nCol: int, nGridLineType: AcGridLineType, pColor: AcadAcCmColor) -> None:
        self._obj.SetGridColor2(nRow, nCol, nGridLineType, pColor())

    def SetGridLineWeight(self, gridLineTypes: AcGridLineType, rowTypes: AcRowType, Lineweight: AcLineWeight) -> None:
        self._obj.SetGridLineWeight(gridLineTypes, rowTypes, Lineweight)

    def SetGridLineWeight2(self, bstrCellStyle: str, gridLineType: AcGridLineType, Lineweight: AcLineWeight) -> None:
        self._obj.SetGridLineWeight2(bstrCellStyle, gridLineType, Lineweight)

    def SetGridVisibility(self, gridLineTypes: AcGridLineType, rowTypes: AcRowType, bVisible: bool) -> None:
        self._obj.SetGridVisibility(gridLineTypes, rowTypes, bVisible)

    def SetGridVisibility2(self, bstrCellStyle: str, gridLineType: AcGridLineType, bValue: bool) -> None:
        self._obj.SetGridVisibility2(bstrCellStyle, gridLineType, bValue)

    def SetRotation(self, bstrCellStyle: str, Rotation: float) -> None:
        self._obj.SetRotation(bstrCellStyle, Rotation)

    def SetTemplateId(self, val: int, option: AcMergeCellStyleOption) -> None:
        self._obj.SetTemplateId(val, option)

    def SetTextHeight(self, rowTypes: AcRowType, TextHeight: float) -> None:
        self._obj.SetTextHeight(rowTypes, TextHeight)

    def SetTextHeight2(self, bstrCellStyle: str, Height: float) -> None:
        self._obj.SetTextHeight2(bstrCellStyle, Height)

    def SetTextStyle(self, rowTypes: AcRowType, bstrName: str) -> None:
        self._obj.SetTextStyle(rowTypes, bstrName)

    def SetTextStyleId(self, bstrCellStyle: str, val: int) -> None:
        self._obj.SetTextStyleId(bstrCellStyle, val)


from ..Base import *
from ..AcadObject import *
from ..Proxy import AccessMode as AccessMode, proxy_property as proxy_property

class AcadPreferences(AppObject):
    def __init__(self, obj) -> None: ...
    Application: AcadApplication
    Display: AcadPreferencesDisplay
    Drafting: AcadPreferencesDrafting
    Files: AcadPreferencesFiles
    OpenSave: AcadPreferencesOpenSave
    Output: AcadPreferencesOutput
    Profiles: AcadPreferencesProfiles
    Selection: AcadPreferencesSelection
    System: AcadPreferencesSystem
    User: AcadPreferencesUser



from ..Base import *
from ..Proxy import *
from .AcadExternalReference import AcadExternalReference as AcadExternalReference

class AcadComparedReference(AcadExternalReference):
    def __init__(self, obj) -> None: ...
    def Copy(self) -> AcadComparedReference: ...



from .Acad3DFace import *
from .Acad3DPolyline import *
from .Acad3DSolid import *
from .AcadArc import *
from .AcadAttribute import *
from .AcadAttributeReference import *
from .AcadBlockReference import *
from .AcadCircle import *
from .AcadComparedReference import *
from .AcadDgnUnderlay import *
from .AcadDim3PointAngular import *
from .AcadDimAligned import *
from .AcadDimAngular import *
from .AcadDimArcLength import *
from .AcadDimDiametric import *
from .AcadDimension import *
from .AcadDimOrdinate import *
from .AcadDimRadial import *
from .AcadDimRadialLarge import *
from .AcadDimRotated import *
from .AcadDwfUnderlay import *
from .AcadEllipse import *
from .AcadExternalReference import *
from .AcadExtrudedSurface import *
from .AcadGeomapImage import *
from .AcadGeoPositionMarker import *
from .AcadHatch import *
from .AcadHelix import *
from .AcadLeader import *
from .AcadLine import *
from .AcadLoftedSurface import *
from .AcadLWPolyline import *
from .AcadMInsertBlock import *
from .AcadMLeader import *
from .AcadMLine import *
from .AcadMtext import *
from .AcadNurbSurface import *
from .AcadOle import *
from .AcadPdfUnderlay import *
from .AcadPlaneSurface import *
from .AcadPoint import *
from .AcadPointCloud import *
from .AcadPointCloudEx import *
from .AcadPolyfaceMesh import *
from .AcadPolygonMesh import *
from .AcadPolyline import *
from .AcadPViewport import *
from .AcadRasterImage import *
from .AcadRay import *
from .AcadRegion import *
from .AcadSection import *
from .AcadShape import *
from .AcadSolid import *
from .AcadSpline import *
from .AcadSubDMesh import *
from .AcadSurface import *
from .AcadSweptSurface import *
from .AcadTable import *
from .AcadText import *
from .AcadTolerance import *
from .AcadTrace import *
from .AcadUnderlay import *
from .AcadWipeout import *
from .AcadXline import *

__all__ = ['Acad3DFace', 'Acad3DPolyline', 'Acad3DSolid', 'AcadArc', 'AcadAttribute', 'AcadAttributeReference', 'AcadBlockReference', 'AcadCircle', 'AcadComparedReference', 'AcadDgnUnderlay', 'AcadDim3PointAngular', 'AcadDimAligned', 'AcadDimAngular', 'AcadDimArcLength', 'AcadDimDiametric', 'AcadDimension', 'AcadDimOrdinate', 'AcadDimRadial', 'AcadDimRadialLarge', 'AcadDimRotated', 'AcadDwfUnderlay', 'AcadEllipse', 'AcadExternalReference', 'AcadExtrudedSurface', 'AcadGeomapImage', 'AcadGeoPositionMarker', 'AcadHatch', 'AcadHelix', 'AcadLeader', 'AcadLine', 'AcadLoftedSurface', 'AcadLWPolyline', 'AcadMInsertBlock', 'AcadMLeader', 'AcadMLine', 'AcadMtext', 'AcadNurbSurface', 'AcadOle', 'AcadPdfUnderlay', 'AcadPlaneSurface', 'AcadPoint', 'AcadPointCloud', 'AcadPointCloudEx', 'AcadPolyfaceMesh', 'AcadPolygonMesh', 'AcadPolyline', 'AcadPViewport', 'AcadRasterImage', 'AcadRay', 'AcadRegion', 'AcadSection', 'AcadShape', 'AcadSolid', 'AcadSpline', 'AcadSubDMesh', 'AcadSurface', 'AcadSweptSurface', 'AcadTable', 'AcadText', 'AcadTolerance', 'AcadTrace', 'AcadUnderlay', 'AcadWipeout', 'AcadXline']

# Names in __all__ with no definition:
#   Acad3DFace
#   Acad3DPolyline
#   Acad3DSolid
#   AcadArc
#   AcadAttribute
#   AcadAttributeReference
#   AcadBlockReference
#   AcadCircle
#   AcadComparedReference
#   AcadDgnUnderlay
#   AcadDim3PointAngular
#   AcadDimAligned
#   AcadDimAngular
#   AcadDimArcLength
#   AcadDimDiametric
#   AcadDimOrdinate
#   AcadDimRadial
#   AcadDimRadialLarge
#   AcadDimRotated
#   AcadDimension
#   AcadDwfUnderlay
#   AcadEllipse
#   AcadExternalReference
#   AcadExtrudedSurface
#   AcadGeoPositionMarker
#   AcadGeomapImage
#   AcadHatch
#   AcadHelix
#   AcadLWPolyline
#   AcadLeader
#   AcadLine
#   AcadLoftedSurface
#   AcadMInsertBlock
#   AcadMLeader
#   AcadMLine
#   AcadMtext
#   AcadNurbSurface
#   AcadOle
#   AcadPViewport
#   AcadPdfUnderlay
#   AcadPlaneSurface
#   AcadPoint
#   AcadPointCloud
#   AcadPointCloudEx
#   AcadPolyfaceMesh
#   AcadPolygonMesh
#   AcadPolyline
#   AcadRasterImage
#   AcadRay
#   AcadRegion
#   AcadSection
#   AcadShape
#   AcadSolid
#   AcadSpline
#   AcadSubDMesh
#   AcadSurface
#   AcadSweptSurface
#   AcadTable
#   AcadText
#   AcadTolerance
#   AcadTrace
#   AcadUnderlay
#   AcadWipeout
#   AcadXline



from __future__ import annotations
from ..VarType import *
import math

class PyGeVector3d(vDoubleArray):
    def __init__(self, *args):
        super().__init__(*args, check_count = 3, set_default = 0.0)

    @property
    def x(self): return self[0]
    @x.setter
    def x(self, value): self[0] = value
    @property
    def y(self): return self[1]
    @y.setter
    def y(self, value): self[1] = value
    @property
    def z(self): return self[2]
    @z.setter
    def z(self, value): self[2] = value

    def __add__(self, vector: PyGeVector3d) -> PyGeVector3d:
        """Сложение двух векторов (Vector + Vector)"""
        if isinstance(vector, PyGeVector3d):
            return PyGeVector3d(self.x + vector.x,self.y + vector.y, self.z + vector.z)
        raise TypeError(f"Неподдерживаемый тип операнда {type(vector)}")


    def __sub__(self, vector: PyGeVector3d) -> PyGeVector3d:
        """Вычитание двух векторов (Vector - Vector)"""
        if isinstance(vector, PyGeVector3d):
            return PyGeVector3d(self.x - vector.x, self.y - vector.y, self.z - vector.z)
        raise TypeError(f"Неподдерживаемый тип операнда {type(vector)}")

    def __mul__(self, scalar: int|float) -> PyGeVector3d:
        """Умножение вектора на скаляр (Vector * Scalar)"""
        if isinstance(scalar, (int, float)):
            return PyGeVector3d(self.x * scalar, self.y * scalar, self.z * scalar)
        raise TypeError(f"Неподдерживаемый тип операнда {type(scalar)}")

    def __rmul__(self, scalar: int|float) -> PyGeVector3d:
        """Зеркальное умножение на скаляр (Scalar * Vector)"""
        return self.__mul__(scalar)

    def __truediv__(self, scalar: int|float) -> PyGeVector3d:
        """Деление вектора на скаляр (Vector / Scalar)"""
        if isinstance(scalar, (int, float)):
            if scalar == 0:
                raise ZeroDivisionError("Деление на ноль невозможно.")
            return PyGeVector3d(self.x / scalar, self.y / scalar, self.z / scalar)
        raise TypeError(f"Неподдерживаемый тип операнда {type(scalar)}")
        
    def __neg__(self) -> PyGeVector3d:
        """Унарный минус (-Vector)"""
        return PyGeVector3d(-self.x, -self.y, -self.z)

    def __iadd__(self, vector) -> PyGeVector3d:
        """Оператор += (Vector += Vector)"""
        if isinstance(vector, PyGeVector3d):
            self.x += vector.x
            self.y += vector.y
            self.z += vector.z
            return self
        raise TypeError(f"Неподдерживаемый тип операнда {type(vector)}")

    def __isub__(self, vector) -> PyGeVector3d:
        """Оператор -= (Vector -= Vector)"""
        if isinstance(vector, PyGeVector3d):
            self.x -= vector.x
            self.y -= vector.y
            self.z -= vector.z
            return self
        raise TypeError(f"Неподдерживаемый тип операнда {type(vector)}")
            
    def __imul__(self, scalar) -> PyGeVector3d:
        """Оператор *= (Vector *= Scalar)"""
        if isinstance(scalar, (int, float)):
            self.x *= scalar
            self.y *= scalar
            self.z *= scalar
            return self
        raise TypeError(f"Неподдерживаемый тип операнда {type(scalar)}")
        
    def __itruediv__(self, scalar) -> PyGeVector3d:
        """Оператор /= (Vector /= Scalar)"""
        if isinstance(scalar, (int, float)):
            if scalar == 0:
                raise ZeroDivisionError("Деление на ноль невозможно.")
            self.x /= scalar
            self.y /= scalar
            self.z /= scalar
            return self
        raise TypeError(f"Неподдерживаемый тип операнда {type(scalar)}")

    def length(self) -> float:
        """Возвращает длину (магнитуду) вектора"""
        return math.sqrt(self.lengthSqrd())
    
    def lengthSqrd(self) -> float:
        """Возвращает квадрат длины вектора"""
        # Можно использовать вместо length() для сравнения длин 2 векторов (быстрее length())
        return self.x**2 + self.y**2 + self.z**2

    def normalize(self) -> None:
        """Нормализует вектор (делает его единичным)"""
        len_val = self.length()
        if len_val == 0:
            raise ZeroDivisionError("Невозможно нормализовать нулевой вектор.")
        self.x /= len_val
        self.y /= len_val
        self.z /= len_val
        # return self

    def normal(self) -> PyGeVector3d:
        """Возвращает новый нормализованный (единичный) вектор"""
        len_val = self.length()
        if len_val == 0:
            raise ZeroDivisionError("Невозможно получить нормаль от нулевого вектора.")
        return PyGeVector3d(self.x / len_val, self.y / len_val, self.z / len_val)
        
    def dotProduct(self, vector: PyGeVector3d) -> float:
        """Скалярное произведение векторов"""
        if isinstance(vector, PyGeVector3d):
            return (self.x * vector.x + 
                    self.y * vector.y + 
                    self.z * vector.z)
        raise TypeError(f"Неподдерживаемый тип операнда {type(vector)}")

    def crossProduct(self, vector: PyGeVector3d) -> PyGeVector3d:
        """Векторное произведение векторов"""
        if isinstance(vector, PyGeVector3d):
            i = self.y * vector.z - self.z * vector.y
            j = self.z * vector.x - self.x * vector.z
            k = self.x * vector.y - self.y * vector.x
            return PyGeVector3d(i, j, k)
        raise TypeError(f"Неподдерживаемый тип операнда {type(vector)}")
        
    def angleTo(self, vector: PyGeVector3d) -> float:
        """Вычисляет угол между двумя векторами в радианах"""
        if isinstance(vector, PyGeVector3d):
            # cos(theta) = (A . B) / (|A| * |B|)
            dot = self.dotProduct(vector)
            len1 = self.length()
            len2 = vector.length()
            if len1 == 0 or len2 == 0:
                return 0.0 # Угол для нулевых векторов не определен
            # Ограничиваем значение в диапазоне [-1, 1] для безопасности acos при ошибках округления
            cosine_angle = max(-1.0, min(dot / (len1 * len2), 1.0))
            return math.acos(cosine_angle)
        raise TypeError(f"Неподдерживаемый тип операнда {type(vector)}")

    def isEqualTo(self, vector: PyGeVector3d, tolerance = 1e-9) -> bool:
        """Проверяет равенство векторов с учетом допуска"""
        if isinstance(vector, PyGeVector3d):
            dx = abs(self.x - vector.x)
            dy = abs(self.y - vector.y)
            dz = abs(self.z - vector.z)
            return dx <= tolerance and dy <= tolerance and dz <= tolerance
        raise TypeError(f"Неподдерживаемый тип операнда {type(vector)}")

    def negate(self) -> None:
        """Инвертирует вектор"""
        self.x = -self.x
        self.y = -self.y
        self.z = -self.z
        # return self
    

    # def __repr__(self):
    #     return f"{self.__class__.__name__}(x={self.x}, y={self.y}, z={self.z})"
    def __str__(self):
        return f"{self.__class__.__name__}(x={self.x}, y={self.y}, z={self.z})"
    
    kIdentity:PyGeVector3d
    kXaxis:PyGeVector3d
    kYaxis:PyGeVector3d
    kZaxis:PyGeVector3d

PyGeVector3d.kIdentity  = PyGeVector3d(0.0, 0.0, 0.0) # Нулевой вектор
PyGeVector3d.kXaxis     = PyGeVector3d(1.0, 0.0, 0.0) # Единичный вектор X
PyGeVector3d.kYaxis     = PyGeVector3d(0.0, 1.0, 0.0) # Единичный вектор Y
PyGeVector3d.kZaxis     = PyGeVector3d(0.0, 0.0, 1.0) # Единичный вектор Z







class PyGeVector2d(vDoubleArray):
    def __init__(self, *args):
        super().__init__(*args, check_count = 2, set_default = 0.0)

    @property
    def x(self): return self[0]
    @x.setter
    def x(self, value): self[0] = value
    @property
    def y(self): return self[1]
    @y.setter
    def y(self, value): self[1] = value

    def __add__(self, vector: PyGeVector2d) -> PyGeVector3d:
        """Сложение двух векторов (Vector + Vector)"""
        if isinstance(vector, PyGeVector2d):
            return PyGeVector2d(self.x + vector.x,self.y + vector.y)
        raise TypeError(f"Неподдерживаемый тип операнда {type(vector)}")


    def __sub__(self, vector: PyGeVector2d) -> PyGeVector2d:
        """Вычитание двух векторов (Vector - Vector)"""
        if isinstance(vector, PyGeVector2d):
            return PyGeVector2d(self.x - vector.x, self.y - vector.y)
        raise TypeError(f"Неподдерживаемый тип операнда {type(vector)}")

    def __mul__(self, scalar: int|float) -> PyGeVector2d:
        """Умножение вектора на скаляр (Vector * Scalar)"""
        if isinstance(scalar, (int, float)):
            return PyGeVector2d(self.x * scalar, self.y * scalar)
        raise TypeError(f"Неподдерживаемый тип операнда {type(scalar)}")

    def __rmul__(self, scalar: int|float) -> PyGeVector2d:
        """Зеркальное умножение на скаляр (Scalar * Vector)"""
        return self.__mul__(scalar)

    def __truediv__(self, scalar: int|float) -> PyGeVector2d:
        """Деление вектора на скаляр (Vector / Scalar)"""
        if isinstance(scalar, (int, float)):
            if scalar == 0:
                raise ZeroDivisionError("Деление на ноль невозможно.")
            return PyGeVector2d(self.x / scalar, self.y / scalar)
        raise TypeError(f"Неподдерживаемый тип операнда {type(scalar)}")
        
    def __neg__(self) -> PyGeVector2d:
        """Унарный минус (-Vector)"""
        return PyGeVector2d(-self.x, -self.y)

    def __iadd__(self, vector) -> PyGeVector2d:
        """Оператор += (Vector += Vector)"""
        if isinstance(vector, PyGeVector2d):
            self.x += vector.x
            self.y += vector.y
            return self
        raise TypeError(f"Неподдерживаемый тип операнда {type(vector)}")

    def __isub__(self, vector) -> PyGeVector2d:
        """Оператор -= (Vector -= Vector)"""
        if isinstance(vector, PyGeVector2d):
            self.x -= vector.x
            self.y -= vector.y
            return self
        raise TypeError(f"Неподдерживаемый тип операнда {type(vector)}")
            
    def __imul__(self, scalar) -> PyGeVector2d:
        """Оператор *= (Vector *= Scalar)"""
        if isinstance(scalar, (int, float)):
            self.x *= scalar
            self.y *= scalar
            return self
        raise TypeError(f"Неподдерживаемый тип операнда {type(scalar)}")
        
    def __itruediv__(self, scalar) -> PyGeVector2d:
        """Оператор /= (Vector /= Scalar)"""
        if isinstance(scalar, (int, float)):
            if scalar == 0:
                raise ZeroDivisionError("Деление на ноль невозможно.")
            self.x /= scalar
            self.y /= scalar
            return self
        raise TypeError(f"Неподдерживаемый тип операнда {type(scalar)}")

    def length(self) -> float:
        """Возвращает длину (магнитуду) вектора"""
        return math.sqrt(self.lengthSqrd())
    
    def lengthSqrd(self) -> float:
        """Возвращает квадрат длины вектора"""
        # Можно использовать вместо length() для сравнения длин 2 векторов (быстрее length())
        return self.x**2 + self.y**2

    def normalize(self) -> PyGeVector2d:
        """Нормализует вектор (делает его единичным)"""
        len_val = self.length()
        if len_val == 0:
            raise ZeroDivisionError("Невозможно нормализовать нулевой вектор.")
        self.x /= len_val
        self.y /= len_val
        return self

    def normal(self) -> PyGeVector2d:
        """Возвращает новый нормализованный (единичный) вектор"""
        len_val = self.length()
        if len_val == 0:
            raise ZeroDivisionError("Невозможно получить нормаль от нулевого вектора.")
        return PyGeVector2d(self.x / len_val, self.y / len_val)
        
    def dotProduct(self, vector: PyGeVector2d) -> float:
        """Скалярное произведение векторов"""
        if isinstance(vector, PyGeVector2d):
            return (self.x * vector.x + 
                    self.y * vector.y)
        raise TypeError(f"Неподдерживаемый тип операнда {type(vector)}")

    def crossProduct(self, vector: PyGeVector2d) -> float:
        """Векторное произведение векторов"""
        if isinstance(vector, PyGeVector2d):
            return self.x * vector.y - self.y * vector.x
        raise TypeError(f"Неподдерживаемый тип операнда {type(vector)}")
        
    def angleTo(self, vector: PyGeVector2d) -> float:
        """Вычисляет угол между двумя векторами в радианах"""
        if isinstance(vector, PyGeVector2d):
            # cos(theta) = (A . B) / (|A| * |B|)
            dot = self.dotProduct(vector)
            len1 = self.length()
            len2 = vector.length()
            if len1 == 0 or len2 == 0:
                return 0.0 # Угол для нулевых векторов не определен
            # Ограничиваем значение в диапазоне [-1, 1] для безопасности acos при ошибках округления
            cosine_angle = max(-1.0, min(dot / (len1 * len2), 1.0))
            return math.acos(cosine_angle)
        raise TypeError(f"Неподдерживаемый тип операнда {type(vector)}")

    def isEqualTo(self, vector: PyGeVector2d, tolerance=1e-9) -> bool:
        """Проверяет равенство векторов с учетом допуска"""
        if isinstance(vector, PyGeVector2d):
            dx = abs(self.x - vector.x)
            dy = abs(self.y - vector.y)
            return dx <= tolerance and dy <= tolerance
        raise TypeError(f"Неподдерживаемый тип операнда {type(vector)}")

    def negate(self) -> PyGeVector2d:
        """Инвертирует вектор"""
        self.x = -self.x
        self.y = -self.y
        return self
    

    # def __repr__(self):
    #     return f"{self.__class__.__name__}(x={self.x}, y={self.y}, z={self.z})"
    def __str__(self):
        return f"{self.__class__.__name__}(x={self.x}, y={self.y}, z={self.z})"
    
    kIdentity:PyGeVector2d # Нулевой вектор
    kXaxis:PyGeVector2d # Единичный вектор X
    kYaxis:PyGeVector2d # Единичный вектор Y

PyGeVector2d.kIdentity  = PyGeVector2d(0.0, 0.0) # Нулевой вектор
PyGeVector2d.kXaxis     = PyGeVector2d(1.0, 0.0) # Единичный вектор X
PyGeVector2d.kYaxis     = PyGeVector2d(0.0, 1.0) # Единичный вектор Y



from ..Base import *
from ..AcadObject import *
from .AcadTextStyle import *

class AcadTextStyles(IAcadObjectCollection):
    def __init__(self, obj) -> None: ...
    def Add(self, Name: str = None) -> AcadTextStyle: ...
    def Item(self, Index: int) -> AcadTextStyle: ...
    def __iter__(self): ...



from ..Base import *
from ..AcadObject import *
from ..Proxy import *
from .AcadDocument import AcadDocument as AcadDocument

class AcadDocuments(IAcadObjectCollection):
    def __init__(self, obj) -> None: ...
    Application: AcadApplication
    Count: int
    def Add(self, Name: str = '') -> AcadDocument: ...
    def Close(self) -> None: ...
    def Item(self, Index: int) -> AcadDocument: ...
    def Open(self, Name: str, ReadOnly: bool = False, Password: Variant = ...) -> AcadDocument: ...
    def __iter__(self): ...



from ..Base import *
from ..AcadObject import *
from ..Proxy import AccessMode as AccessMode, proxy_property as proxy_property

class AcadPreferencesDrafting(AppObject):
    def __init__(self, obj) -> None: ...
    AlignmentPointAcquisition: AcAlignmentPointAcquisition
    Application: AcadApplication
    AutoSnapAperture: bool
    AutoSnapApertureSize: int
    AutoSnapMagnet: bool
    AutoSnapMarker: bool
    AutoSnapMarkerColor: AcColor
    AutoSnapMarkerSize: int
    AutoSnapToolTip: bool
    AutoTrackTooltip: bool
    FullScreenTrackingVector: bool
    PolarTrackingVector: bool



from __future__ import annotations
from ..Base import *
from ..Proxy import *
from ..AcadEntity import AcadEntity



class AcadShape(AcadEntity):
    def __init__(self, obj) -> None: super().__init__(obj)

    InsertionPoint: PyGePoint3d = proxy_property('PyGePoint3d','InsertionPoint',AccessMode.ReadWrite)
    Name: str = proxy_property(str,'Name',AccessMode.ReadWrite)
    Normal: PyGeVector3d = proxy_property('PyGeVector3d','Normal',AccessMode.ReadWrite)
    ObliqueAngle: float = proxy_property(float,'ObliqueAngle',AccessMode.ReadWrite)
    Rotation: float = proxy_property(float,'Rotation',AccessMode.ReadWrite)
    ScaleFactor: float = proxy_property(float,'ScaleFactor',AccessMode.ReadWrite)
    Thickness: float = proxy_property(float,'Thickness',AccessMode.ReadWrite)

    def Copy(self) -> AcadShape:
        return AcadShape(self._obj.Copy())




from __future__ import annotations
from ..Base import *
from ..Proxy import *
from ..AcadEntity import AcadEntity



class AcadPViewport(AcadEntity):
    def __init__(self, obj) -> None: super().__init__(obj)

    ArcSmoothness: int = proxy_property(int,'ArcSmoothness',AccessMode.ReadWrite)
    Center: PyGePoint3d = proxy_property('PyGePoint3d','Center',AccessMode.ReadWrite)
    Clipped: bool = proxy_property(bool,'Clipped',AccessMode.ReadOnly)
    CustomScale: float = proxy_property(float,'CustomScale',AccessMode.ReadWrite)
    Direction: PyGeVector3d = proxy_property('PyGeVector3d','Direction',AccessMode.ReadWrite)
    DisplayLocked: bool = proxy_property(bool,'DisplayLocked',AccessMode.ReadWrite)
    DisplayLocked: bool = proxy_property(bool,'DisplayLocked',AccessMode.ReadWrite)
    GridOn: bool = proxy_property(bool,'GridOn',AccessMode.ReadWrite)
    HasSheetView: bool = proxy_property(bool,'HasSheetView',AccessMode.ReadOnly)
    Height: float = proxy_property(float,'Height',AccessMode.ReadWrite)
    LabelBlockId: int = proxy_property(int,'LabelBlockId',AccessMode.ReadWrite)
    LayerPropertyOverrides: bool = proxy_property(bool,'LayerPropertyOverrides',AccessMode.ReadOnly)
    LensLength: float = proxy_property(float,'LensLength',AccessMode.ReadWrite)
    ModelView: AcadView = proxy_property('AcadView','ModelView',AccessMode.ReadWrite)
    ShadePlot: AcShadePlot = proxy_property('AcShadePlot','ShadePlot',AccessMode.ReadWrite)
    SheetView: AcadView = proxy_property('AcadView','SheetView',AccessMode.ReadWrite)
    SnapBasePoint: PyGePoint2d = proxy_property('PyGePoint2d','SnapBasePoint',AccessMode.ReadWrite)
    SnapOn: bool = proxy_property(bool,'SnapOn',AccessMode.ReadWrite)
    SnapRotationAngle: float = proxy_property(float,'SnapRotationAngle',AccessMode.ReadWrite)
    StandardScale: AcViewportScale = proxy_property('AcViewportScale','StandardScale',AccessMode.ReadWrite)
    StandardScale2: int = proxy_property(int,'StandardScale2',AccessMode.ReadWrite)
    Target: PyGePoint3d = proxy_property('PyGePoint3d','Target',AccessMode.ReadWrite)
    TwistAngle: float = proxy_property(float,'TwistAngle',AccessMode.ReadWrite)
    UCSIconAtOrigin: bool = proxy_property(bool,'UCSIconAtOrigin',AccessMode.ReadWrite)
    UCSIconOn: bool = proxy_property(bool,'UCSIconOn',AccessMode.ReadWrite)
    UCSPerViewport: bool = proxy_property(bool,'UCSPerViewport',AccessMode.ReadWrite)
    ViewportOn: bool = proxy_property(bool,'ViewportOn',AccessMode.ReadWrite)
    VisualStyle: int = proxy_property(int,'VisualStyle',AccessMode.ReadWrite)
    Width: float = proxy_property(float,'Width',AccessMode.ReadWrite)

    def Copy(self) -> AcadPViewport:
        return AcadPViewport(self._obj.Copy())
        
    def GetGridSpacing(self) -> tuple:
        XSpacing, YSpacing = self._obj.GetGridSpacing()
        return XSpacing, YSpacing
        
    def GetSnapSpacing(self) -> tuple:
        XSpacing, YSpacing = self._obj.GetSnapSpacing()
        return XSpacing, YSpacing
        
    def SetGridSpacing(self, XSpacing: float, YSpacing: float) -> None:
        self._obj.SetGridSpacing(XSpacing, YSpacing)
        
    def SetSnapSpacing(self, XSpacing: float, YSpacing: float) -> None:
        self._obj.SetSnapSpacing(XSpacing, YSpacing)
        
    def SyncModelView(self) -> None:
        self._obj.SyncModelView()



from ..Base import *
from ..Proxy import *
from .AcadRasterImage import AcadRasterImage as AcadRasterImage

class AcadWipeout(AcadRasterImage):
    def __init__(self, obj) -> None: ...
    def Copy(self) -> AcadWipeout: ...



from ..Base import *
from ..Proxy import *

class AcadSectionTypeSettings(AppObject):
    def __init__(self, obj) -> None: ...
    BackgroundLinesColor: AcadAcCmColor
    BackgroundLinesHiddenLine: bool
    BackgroundLinesLayer: str
    BackgroundLinesLinetypeScale: float
    BackgroundLinesLineweight: AcLineWeight
    BackgroundLinesPlotStyleName: str
    BackgroundLinesVisible: bool
    CurveTangencyLinesColor: AcadAcCmColor
    CurveTangencyLinesLayer: str
    CurveTangencyLinesLinetype: str
    CurveTangencyLinesLinetypeScale: float
    CurveTangencyLinesLineweight: AcLineWeight
    CurveTangencyLinesPlotStyleName: str
    CurveTangencyLinesVisible: bool
    DestinationBlock: Variant
    DestinationFile: str
    ForegroundLinesColor: AcadAcCmColor
    ForegroundLinesEdgeTransparency: int
    ForegroundLinesFaceTransparency: int
    ForegroundLinesHiddenLine: bool
    ForegroundLinesLayer: str
    ForegroundLinesLinetype: str
    ForegroundLinesLinetypeScale: float
    ForegroundLinesLineweight: AcLineWeight
    ForegroundLinesPlotStyleName: str
    ForegroundLinesVisible: bool
    GenerationOptions: AcSectionGeneration
    IntersectionBoundaryColor: AcadAcCmColor
    IntersectionBoundaryDivisionLines: bool
    IntersectionBoundaryLayer: str
    IntersectionBoundaryLinetype: str
    IntersectionBoundaryLinetypeScale: float
    IntersectionBoundaryLineweight: AcLineWeight
    IntersectionBoundaryPlotStyleName: str
    IntersectionBoundaryVisible: bool
    IntersectionFillColor: AcadAcCmColor
    IntersectionFillFaceTransparency: int
    IntersectionFillHatchAngle: float
    IntersectionFillHatchPatternName: str
    IntersectionFillHatchPatternType: AcPatternType
    IntersectionFillHatchScale: float
    IntersectionFillHatchSpacing: float
    IntersectionFillLayer: str
    IntersectionFillLinetype: str
    IntersectionFillLinetypeScale: float
    IntersectionFillLineweight: AcLineWeight
    IntersectionFillPlotStyleName: str
    IntersectionFillVisible: bool
    SourceObjects: Variant



from __future__ import annotations
from ..Base import *
from ..Proxy import *
from ..AcadEntity import AcadEntity



class AcadEllipse(AcadEntity):
    def __init__(self, obj) -> None: super().__init__(obj)

    Area: float = proxy_property(float,'Area',AccessMode.ReadOnly)
    Center: PyGePoint3d = proxy_property('PyGePoint3d','Center',AccessMode.ReadWrite)
    EndAngle: float = proxy_property(float,'EndAngle',AccessMode.ReadWrite)
    EndParameter: float = proxy_property(float,'EndParameter',AccessMode.ReadWrite)
    EndPoint: PyGePoint3d = proxy_property('PyGePoint3d','EndPoint',AccessMode.ReadWrite)
    MajorAxis: PyGeVector3d = proxy_property('PyGeVector3d','MajorAxis',AccessMode.ReadWrite)
    MajorRadius: float = proxy_property(float,'MajorRadius',AccessMode.ReadWrite)
    MinorAxis: PyGeVector3d = proxy_property('PyGeVector3d','MajorAxis',AccessMode.ReadOnly)
    MinorRadius: float = proxy_property(float,'MinorRadius',AccessMode.ReadWrite)
    Normal: PyGeVector3d = proxy_property('PyGeVector3d','Normal',AccessMode.ReadWrite)
    RadiusRatio: float = proxy_property(float,'RadiusRatio',AccessMode.ReadWrite)
    StartAngle: float = proxy_property(float,'StartAngle',AccessMode.ReadWrite)
    StartParameter: float = proxy_property(float,'StartParameter',AccessMode.ReadWrite)
    StartPoint: PyGePoint3d = proxy_property('PyGePoint3d','StartPoint',AccessMode.ReadWrite)

    def Copy(self) -> AcadEllipse:
        return AcadEllipse(self._obj.Copy())

    def Offset(self, Distance: float) -> AcadEllipse:
        return AcadEllipse(self._obj.Offset(Distance))



from ..Base import *
from ..Proxy import *
from .AcadPlotConfiguration import *

class AcadLayout(AcadPlotConfiguration):
    def __init__(self, obj) -> None: ...
    Block: AcadBlock
    TabOrder: int



import pytest
import time
import math
from typing import Generator
from simpleautocad import *
# import pandas as pd
from colorama import init, Fore, Back, Style
init(autoreset=True)

OFFSET_DIR:PyGeVector3d = PyGeVector3d.kXaxis
OFFSET_COUNT:int = 0
OFFSET_DIST_X:int = 100
OFFSET_DIST_Y:int = 100
OFFSET_DIST_Z:int = 0

def init_point():
    global OFFSET_COUNT, OFFSET_DIR, OFFSET_DIST_X, OFFSET_DIST_Y, OFFSET_DIST_Z
    init_pt = PyGePoint3d(OFFSET_COUNT*OFFSET_DIR.x*OFFSET_DIST_X)
    OFFSET_COUNT += 1
    return init_pt

@pytest.fixture(scope="session")
def acad_app() -> Generator[AutoCAD,None,None]:
    # if com_server_is_running(GetProgID(AcadApplication)):
    #     pytest.exit("Перед запуском теста необходимо закрыть AutoCAD")
    print("\n--- Продключение к AutoCAD.Application ---")
    try:
        acad = AutoCAD()
    except Exception as e:
        pytest.fail(f"Не удалось запустить AutoCAD: {e}")
    # Ожидание загрузки
    # _ = acad.VBE.MainWindow.Visible
    # timeout = 20
    # start_time = time.time()
    # while time.time() - start_time < timeout:
    #     try:
    #         # Попытка доступа к свойству, которое требует полной инициализации
    #         print(".")
    #         _ = acad.VBE.MainWindow.Visible 
    #         print("\nAutoCAD готов к работе.")
    #         break
    #     except pythoncom.com_error:
    #         # Пока не готов, ждем
    #         time.sleep(0.2)
    #         # print("\n\tПродключение к AutoCAD.Application .",end=" ")
    # else:
    #     pytest.fail("Время ожидания запуска AutoCAD истекло.")
    time.sleep(1.2)
    acad.Visible = True
    yield acad
    # acad.Visible = True
    # time.sleep(0.2)
    # acad.ActiveDocument.Regen(AcRegenType.acAllViewports)
    # acad.ZoomAll()
    # time.sleep(5)
    # print(f"Сохранение документа...")
    # acad.ActiveDocument.SaveAs(r'Y:\simpleautocad_test.dwg',AcSaveAsType.ac2007_dwg)
    # acad.Quit() # Закомментировать, чтобы окно не закрывалось сразу после тестов
    print("\n--- Отключение AutoCAD.Application ---")
    del acad

@pytest.fixture(scope="function")
def acad_active_document(acad_app:AutoCAD) -> Generator[AcadDocument,None,None]:
    time.sleep(0.2)
    doc = acad_app.ActiveDocument
    assert doc.__class__ == AcadDocument
    yield doc

@pytest.fixture(scope="function")
def acad_model_space(acad_active_document:AcadDocument) -> Generator[AcadModelSpace,None,None]:
    # Убедимся, что мы в ModelSpace
    acad_active_document.ActiveSpace = AcActiveSpace.acModelSpace
    ms = acad_active_document.ModelSpace
    assert ms.__class__ == AcadModelSpace
    yield ms
    # Очистка чертежа после каждого теста
    objects_to_delete = [obj for obj in ms]
    for obj in objects_to_delete:
        obj.Delete()
    acad_active_document.Regen(AcRegenType.acAllViewports)
    print("--- Очистка ModelSpace после теста ---")

@pytest.fixture(scope="function")
def object_AcadCircle(acad_model_space:AcadModelSpace) -> Generator[AcadCircle,None,None]:
    print('\n')
    object_class = AcadCircle
    pt = init_point()
    pt[0]+=10
    Center = PyGePoint3d(pt.x + OFFSET_DIST_X / 2, pt.y + OFFSET_DIST_Y / 2, pt.z + OFFSET_DIST_Z / 2)
    Radius = OFFSET_DIST_X / 2 - 10
    prompt = f"Установка свойств {object_class}\n\
{Fore.CYAN}Center{Fore.WHITE} = {Center.__class__.__name__}({Center})\n\
{Fore.CYAN}Radius{Fore.WHITE} = {Radius.__class__.__name__}({Radius})\n\
"
    print(prompt)
    obj = acad_model_space.AddCircle(Center, Radius)
    assert obj.__class__ is object_class
    print(f"{obj} {obj.__class__.__name__} добавлен в {acad_model_space.Name}")
    yield obj
    # print(f'\nУдаление {obj}')
    # obj.Delete()

@pytest.fixture(scope="function")
def object_Acad3DFace(acad_model_space:AcadModelSpace) -> Generator[Acad3DFace,None,None]:
    print('\n')
    object_class = Acad3DFace
    pt = init_point()
    pt[0]+=10
    Point1 = PyGePoint3d(5 + pt.x,5+pt.y,pt.z)
    Point2 = PyGePoint3d(pt.x+OFFSET_DIST_X-5,pt.y+5,pt.z)
    Point3 = PyGePoint3d(pt.x+OFFSET_DIST_X-5,pt.y+OFFSET_DIST_Y-5,pt.z)
    Point4 = PyGePoint3d(pt.x+5,pt.y+OFFSET_DIST_Y-5,pt.z)
    prompt = f"Установка свойств {object_class}\n\
{Fore.CYAN}Point1{Fore.WHITE} = {Point1.__class__.__name__}({Point1})\n\
{Fore.CYAN}Point2{Fore.WHITE} = {Point2.__class__.__name__}({Point2})\n\
{Fore.CYAN}Point3{Fore.WHITE} = {Point3.__class__.__name__}({Point3})\n\
{Fore.CYAN}Point4{Fore.WHITE} = {Point4.__class__.__name__}({Point4})\n\
"
    print(prompt)
    obj = acad_model_space.Add3DFace(Point1, Point2, Point3, Point4)
    assert obj.__class__ is object_class
    print(f"{obj} {obj.__class__.__name__} добавлен в {acad_model_space.Name}")
    yield obj
    # print(f'\nУдаление {obj}')
    # obj.Delete()
    
@pytest.fixture(scope="function")
def object_AcadPolygonMesh(acad_model_space:AcadModelSpace) -> Generator[AcadPolygonMesh,None,None]:
    print('\n')
    object_class = AcadPolygonMesh
    pt = init_point()
    pt[0]+=10
    M = 4
    N = 4
    PoinsMatrix = vDoubleArray(
        [pt.x,0,0,                       pt.x+OFFSET_DIST_X/4,0,10,                       pt.x+OFFSET_DIST_X/2,0,0,                       pt.x+OFFSET_DIST_X,0,1,
        pt.x,pt.y+OFFSET_DIST_Y/4,0,    pt.x+OFFSET_DIST_X/4,pt.y+OFFSET_DIST_Y/4,10,    pt.x+OFFSET_DIST_X/2,pt.y+OFFSET_DIST_Y/4,0,    pt.x+OFFSET_DIST_X,pt.y+OFFSET_DIST_Y/4,10,
        pt.x,pt.y+OFFSET_DIST_Y/2,0,    pt.x+OFFSET_DIST_X/4,pt.y+OFFSET_DIST_Y/2,-10,    pt.x+OFFSET_DIST_X/2,pt.y+OFFSET_DIST_Y/2,0,    pt.x+OFFSET_DIST_X,pt.y+OFFSET_DIST_Y/2,0,
        pt.x,pt.y+OFFSET_DIST_Y,0,      pt.x+OFFSET_DIST_X/4,pt.y+OFFSET_DIST_Y,10,      pt.x+OFFSET_DIST_X/2,pt.y+OFFSET_DIST_Y,0,      pt.x+3*OFFSET_DIST_X/4,pt.y+3*OFFSET_DIST_Y/4,0],
        check_count=M*N*3
    )
    
    prompt = f"Установка свойств {object_class}\n\
{Fore.CYAN}M{Fore.WHITE} = {M.__class__.__name__}({M})\n\
{Fore.CYAN}N{Fore.WHITE} = {N.__class__.__name__}({N})\n\
{Fore.CYAN}PointsMatrix{Fore.WHITE} = {PoinsMatrix.__class__.__name__}({PoinsMatrix})\n\
"
    print(prompt)
    obj = acad_model_space.Add3DMesh(M,N,PoinsMatrix)
    assert obj.__class__ is object_class
    print(f"{object_class} добавлен в {acad_model_space.Name}")
    yield obj
    # print(f'\nУдаление {obj}')
    # obj.Delete()
    
@pytest.fixture(scope="function")
def object_Acad3DPolyline(acad_model_space:AcadModelSpace) -> Generator[Acad3DPolyline,None,None]:
    print('\n')
    object_class = Acad3DPolyline
    pt = init_point()
    pt[0]+=10
    PointsArray = PyGePoint3dArray(0,0,0,10,10,10,15,-15,15,13,5,-4)
    
    prompt = f"Установка свойств {object_class}\n\
{Fore.CYAN}PointsArray{Fore.WHITE} = {PointsArray.__class__.__name__}({PointsArray})\n"
    print(prompt)
    obj = acad_model_space.Add3DPoly(PointsArray)
    assert obj.__class__ is object_class
    print(f"{object_class} добавлен в {acad_model_space.Name}")
    yield obj
    # print(f'\nУдаление {obj}')
    # obj.Delete()
    
@pytest.fixture(scope="function")
def object_AddArc(acad_model_space:AcadModelSpace) -> Generator[AcadArc,None,None]:
    print('\n')
    object_class = AcadArc
    pt = init_point()
    pt[0]+=10
    Center = PyGePoint3d(pt.x + OFFSET_DIST_X / 2, pt.y + OFFSET_DIST_Y / 2, pt.z + OFFSET_DIST_Z / 2)
    Radius = OFFSET_DIST_X / 2 - 10
    StartAngle = math.pi / 2
    EndAngle = math.pi * 2
    prompt = f"Установка свойств {object_class}\n\
{Fore.CYAN}Center{Fore.WHITE} = {Center.__class__.__name__}({Center})\n\
{Fore.CYAN}Radius{Fore.WHITE} = {Radius.__class__.__name__}({Radius})\n\
{Fore.CYAN}StartAngle{Fore.WHITE} = {StartAngle.__class__.__name__}({StartAngle})\n\
{Fore.CYAN}EndAngle{Fore.WHITE} = {EndAngle.__class__.__name__}({EndAngle})\n\
"
    print(prompt)
    obj = acad_model_space.AddArc(Center, Radius, StartAngle, EndAngle)
    assert obj.__class__ is object_class
    print(f"{object_class} добавлен в {acad_model_space.Name}")
    yield obj
    # print(f'\nУдаление {obj}')
    # obj.Delete()
    
@pytest.fixture(scope="function")
def object_AcadAttribute(acad_model_space:AcadModelSpace) -> Generator[AcadAttribute,None,None]:
    print('\n')
    object_class = AcadAttribute
    pt = init_point()
    pt[0]+=10
    Height = 10
    Mode = AcAttributeMode.acAttributeModeMultipleLine
    Prompt = 'Описание атрибута'
    InsertionPoint = PyGePoint3d(pt.x+OFFSET_DIST_X/2,pt.y+OFFSET_DIST_Y/2,pt.z+OFFSET_DIST_Z/2)
    Tag = '_ТЕГ_'
    Value = 'Значение'
    prompt = f"Установка свойств {object_class}\n\
{Fore.CYAN}Height{Fore.WHITE} = {Height.__class__.__name__}({Height})\n\
{Fore.CYAN}Mode{Fore.WHITE} = {Mode.__class__.__name__}({Mode})\n\
{Fore.CYAN}Prompt{Fore.WHITE} = {Prompt.__class__.__name__}({Prompt})\n\
{Fore.CYAN}InsertionPoint{Fore.WHITE} = {InsertionPoint.__class__.__name__}({InsertionPoint})\n\
{Fore.CYAN}Tag{Fore.WHITE} = {Fore.CYAN}{Tag.__class__.__name__}{Fore.WHITE}({Tag})\n\
{Fore.CYAN}Value{Fore.WHITE} = {Value.__class__.__name__}{Fore.WHITE}({Value})\n\
"
    print(prompt)
    obj = acad_model_space.AddAttribute(Height,Mode,Prompt,InsertionPoint,Tag,Value)
    assert obj.__class__ is object_class
    print(f"{object_class} добавлен в {acad_model_space.Name}")
    yield obj
    # print(f'\nУдаление {obj}')
    # obj.Delete()

@pytest.fixture(scope="function")
def object_AddMInsertBlock(acad_model_space:AcadModelSpace) -> Generator[AcadBlock,None,None]:
    bl = acad_model_space.Document.Database.Blocks.Add(PyGePoint3d(),'TestBlock')
    bl.AddArc(PyGePoint3d(),15,math.pi,math.pi/3)
    print('\n')
    object_class = AcadMInsertBlock
    pt = init_point()
    pt[0]+=10
    InsertionPoint = pt
    Name = 'TestBlock'
    XScale= 1.0
    YScale= 1.2 
    ZScale= 1.4
    Rotation= math.pi/2
    NumRows= 2
    NumColumns= 3
    RowSpacing= 10.10
    ColumnSpacing = 20.20
    Password= vObjectEmpty
    prompt = f"Установка свойств {object_class}\n\
{Fore.CYAN}InsertionPoint{Fore.WHITE:<30} = {Fore.CYAN}{InsertionPoint.__class__.__name__}{Fore.WHITE}({InsertionPoint})\n\
{Fore.CYAN}Name{Fore.WHITE:<30} = {Fore.CYAN}{Name.__class__.__name__}{Fore.WHITE}({Name})\n\
{Fore.CYAN}XScale{Fore.WHITE:<30} = {Fore.CYAN}{XScale.__class__.__name__}{Fore.WHITE}({XScale})\n\
{Fore.CYAN}YScale{Fore.WHITE:<30} = {Fore.CYAN}{YScale.__class__.__name__}{Fore.WHITE}({YScale})\n\
{Fore.CYAN}ZScale{Fore.WHITE:<30} = {Fore.CYAN}{ZScale.__class__.__name__}{Fore.WHITE}({ZScale})\n\
{Fore.CYAN}Rotation{Fore.WHITE:<30} = {Fore.CYAN}{Rotation.__class__.__name__}{Fore.WHITE}({Rotation})\n\
{Fore.CYAN}NumRows{Fore.WHITE:<30} = {Fore.CYAN}{NumRows.__class__.__name__}{Fore.WHITE}({NumRows})\n\
{Fore.CYAN}NumColumns{Fore.WHITE:<30} = {Fore.CYAN}{NumColumns.__class__.__name__}{Fore.WHITE}({NumColumns})\n\
{Fore.CYAN}RowSpacing{Fore.WHITE:<30} = {Fore.CYAN}{RowSpacing.__class__.__name__}{Fore.WHITE}({RowSpacing})\n\
{Fore.CYAN}ColumnSpacing{Fore.WHITE:<30} = {Fore.CYAN}{ColumnSpacing.__class__.__name__}{Fore.WHITE}({ColumnSpacing})\n\
{Fore.CYAN}Password{Fore.WHITE:<30} = {Fore.CYAN}{Password.__class__.__name__}{Fore.WHITE}({Password})\n\
"
    print(prompt)
    obj = acad_model_space.AddMInsertBlock(InsertionPoint,Name,XScale,YScale,ZScale,Rotation,NumRows,NumColumns,RowSpacing,ColumnSpacing,Password)
    assert obj.__class__ == object_class
    print(f"{obj} {obj.__class__.__name__} добавлен в {acad_model_space.Name}")
    yield obj
    print(f'\nУдаление {obj}')
    # obj.Delete()


from ..Base import *
from ..Proxy import *
from ..AcadEntity import AcadEntity as AcadEntity

class Acad3DPolyline(AcadEntity):
    def __init__(self, obj) -> None: ...
    Closed: bool
    Coordinates: PyGePoint3dArray
    Length: float
    Type: Ac3DPolylineType
    def Coordinate(self, Index: int) -> PyGePoint3d: ...
    def Copy(self) -> Acad3DPolyline: ...
    def AppendVertex(self, Point: PyGePoint3d) -> None: ...
    def Explode(self) -> vObjectArray: ...



from ..Base import *
from ..AcadObject import *
from ..Proxy import AccessMode as AccessMode, proxy_property as proxy_property

class AcadGroups(IAcadObjectCollection):
    def __init__(self, obj) -> None: ...
    def Add(self, Name: str) -> AcadGroup: ...
    def Item(self, Index: int | str) -> AcadGroup: ...
    def __iter__(self): ...



from ..Base import *
from ..Proxy import *
from ..AcadObject import *

class AcadIDPair(AppObject):
    def __init__(self, obj) -> None: ...
    Application: AcadApplication
    IsCloned: bool
    IsOwnerXlated: bool
    IsPrimary: bool
    Key: int



from ..Base import *
from ..Proxy import *
from ...Types.Ac import *
from .AcadApplication import AcadApplication as AcadApplication

class AcadSecurityParams(AppObject):
    def __init__(self, obj) -> None: ...
    Action: AcadSecurityParamsType
    Algorithm: AcadSecurityParamsConstants
    Comment: str
    Issuer: str
    KeyLength: int
    Password: str
    ProviderName: str
    ProviderType: int
    SerialNumber: str
    Subject: str
    TimeServer: str



from __future__ import annotations
from ..Base import *
from ..Proxy import proxy_property, AccessMode
from ..AcadObject import *



class AcadTextStyle(AcadObject):
    def __init__(self, obj) -> None: super().__init__(obj)

    BigFontFile: str = proxy_property(str,'BigFontFile',AccessMode.ReadWrite)
    FontFile: str = proxy_property(str,'FontFile',AccessMode.ReadWrite)
    Height: float = proxy_property(float,'Height',AccessMode.ReadWrite)
    LastHeight: float = proxy_property(float,'LastHeight',AccessMode.ReadWrite)
    Name: str = proxy_property(str,'Name',AccessMode.ReadOnly)
    ObliqueAngle: float = proxy_property(float,'ObliqueAngle',AccessMode.ReadWrite)
    TextGenerationFlag: AcTextGenerationFlag = proxy_property('AcTextGenerationFlag','TextGenerationFlag',AccessMode.ReadWrite)
    Width: float = proxy_property(float,'Width',AccessMode.ReadWrite)

    def GetFont(self) -> tuple[str,bool,bool,int,int]:
        Typeface, Bold, Italic, CharSet, PitchAndFamily = self._obj.GetFont()
        return Typeface, Bold, Italic, CharSet, PitchAndFamily

    def SetFont(self, Typeface: str, Bold: bool, Italic: bool, CharSet: int, PitchAndFamily: int) -> None:
        self._obj.SetFont(Typeface, Bold, Italic, CharSet, PitchAndFamily)



from ..Base import *
from ..AcadObject import *
from ..Proxy import AccessMode as AccessMode, proxy_property as proxy_property

class AcadHyperlink(AppObject):
    def __init__(self, obj) -> None: ...
    Application: AcadApplication
    URL: str
    URLDescription: str
    URLNamedLocation: str
    def Delete(self) -> None: ...



from __future__ import annotations
from ..Base import *
from ..Proxy import *
from ..AcadObject import *



class AcadLayers(IAcadObjectCollection):
    def __init__(self, obj) -> None: super().__init__(obj)

    # Application: AcadApplication = proxy_property('AcadApplication','Application',AccessMode.ReadOnly)
    # Count: int = proxy_property(int,'Count',AccessMode.ReadOnly)
    # Document: AcadDocument = proxy_property('AcadDocument','Document',AccessMode.ReadOnly)
    # Handle: int = proxy_property(int,'Handle',AccessMode.ReadOnly)
    # HasExtensionDictionary: bool = proxy_property(bool,'HasExtensionDictionary',AccessMode.ReadOnly)
    # ObjectID: int = proxy_property(int,'ObjectID',AccessMode.ReadOnly)
    # ObjectName: str = proxy_property(str,'ObjectName',AccessMode.ReadOnly)
    # OwnerID: int = proxy_property(int,'OwnerID',AccessMode.ReadOnly)

    def Add(self, Name: str) -> AcadLayer: 
        return AcadLayer(self._obj.Add(Name))

    def Item(self, Index: int | str) -> AcadLayer:
        return AcadLayer(self._obj.Item(Index))

    def __iter__(self):
        for item in self._obj:
            obj = AcadLayer(item)
            yield obj




from ..Base import *
from ..Proxy import *
from ..AcadEntity import AcadEntity as AcadEntity
from .AcadLeader import AcadLeader as AcadLeader
from _typeshed import Incomplete

class AcadMLeader(AcadEntity):
    leaderLineIndex: Incomplete
    def __init__(self, obj, leaderLineIndex: int = 0) -> None: ...
    ArrowheadBlock: str
    ArrowheadSize: float
    ArrowheadType: AcDimArrowheadType
    BlockConnectionType: AcBlockConnectionType
    BlockScale: int
    ContentBlockName: str
    ContentBlockType: AcPredefBlockType
    ContentType: AcMLeaderContentType
    DogLegged: bool
    DoglegLength: float
    LandingGap: float
    LeaderCount: int
    LeaderLineColor: AcadAcCmColor
    LeaderLinetype: str
    LeaderLineWeight: AcLineWeight
    LeaderType: AcMLeaderType
    Normal: PyGeVector3d
    ScaleFactor: float
    StyleName: str
    TextAttachmentDirection: AcTextAttachmentDirection
    TextBackgroundFill: bool
    TextBottomAttachmentType: AcVerticalTextAttachmentType
    TextDirection: AcDrawingDirection
    TextFrameDisplay: bool
    TextHeight: float
    TextJustify: AcAttachmentPoint
    TextLeftAttachmentType: AcTextAttachmentType
    TextLineSpacingDistance: float
    TextLineSpacingFactor: float
    TextLineSpacingStyle: AcLineSpacingStyle
    TextRightAttachmentType: AcTextAttachmentType
    TextRotation: float
    TextString: str
    TextStyleName: str
    TextTopAttachmentType: AcVerticalTextAttachmentType
    TextWidth: float
    Type: AcLeaderType
    def AddLeader(self) -> AcadLeader: ...
    def AddLeaderLine(self, leaderIndex: int, pointArray: PyGePoint3dArray) -> int: ...
    def AddLeaderLineEx(self, pointArray: PyGePoint3dArray) -> int: ...
    def Evaluate(self) -> None: ...
    def GetBlockAttributeValue(self, attdefId: int) -> str: ...
    def GetDoglegDirection(self, leaderIndex: int) -> PyGeVector3d: ...
    def GetLeaderIndex(self, leaderLineIndex: int) -> int: ...
    def GetLeaderLineIndexes(self, leaderIndex: int) -> vDoubleArray: ...
    def GetLeaderLineVertices(self, leaderLineIndex: int) -> PyGePoint3dArray: ...
    def GetVertexCount(self, leaderLineIndex: int) -> int: ...
    def RemoveLeader(self, leaderIndex: int) -> None: ...
    def RemoveLeaderLine(self, leaderLineIndex: int) -> None: ...
    def SetBlockAttributeValue(self, attdefId: int, value: str) -> None: ...
    def SetDoglegDirection(self, leaderIndex: int, dirVec: vDoubleArray) -> None: ...
    def SetLeaderLineVertices(self, leaderLineIndex: int, pointArray: PyGePoint3dArray) -> None: ...



from __future__ import annotations
from ..Base import *
from ..Proxy import *
from ..AcadObject import *
from ..AcadEntity import *



class AcadSortentsTable(AcadObject):
    def __init__(self, obj) -> None: super().__init__(obj)

    def Block(self) -> AcadBlock:
        return AcadBlock(self._obj.GetFont())

    def GetFullDrawOrder(self, honorSortentsSysvar: bool) -> vObjectArray:
        Objects = self._obj.GetFullDrawOrder(honorSortentsSysvar)
        return vObjectArray(Objects)

    def GetRelativeDrawOrder(self, honorSortentsSysvar: bool) -> vObjectArray:
        Objects = self._obj.GetRelativeDrawOrder(honorSortentsSysvar)
        return vObjectArray(Objects)

    def MoveAbove(self, Target: AcadEntity) -> vObjectArray:
        Objects = self._obj.MoveAbove(Target())
        return vObjectArray(Objects)

    def MoveBelow(self, Target: AcadEntity) -> vObjectArray:
        Objects = self._obj.MoveBelow(Target())
        return vObjectArray(Objects)

    def MoveToBottom(self, Objects: vObjectArray) -> None:
        self._obj.MoveToBottom(Objects())

    def MoveToTop(self, Objects: vObjectArray) -> None:
        self._obj.MoveToTop(Objects())

    def SetRelativeDrawOrder(self, Objects: vObjectArray) -> None:
        self._obj.SetRelativeDrawOrder(Objects())

    def SwapOrder(self, Object1: AcadEntity, Object2: AcadEntity) -> None:
        self._obj.SwapOrder(Object1(), Object2())



from ..Base import *
from ..Proxy import *
from .AcadSurface import AcadSurface as AcadSurface

class AcadPlaneSurface(AcadSurface):
    def __init__(self, obj) -> None: ...
    def Copy(self) -> AcadPlaneSurface: ...
